@@ -1,4 +1,5 @@
-// TeXFont.cpp
+// -*- C++ -*-
+// TeXFont.h
 //
 // Part of KDVI - A DVI previewer for the KDE desktop environemt 
 //
@@ -8,11 +9,8 @@
 #ifndef _TEXFONT_H
 #define _TEXFONT_H
 
-// Add header files alphabetically
-
 #include "TeXFontDefinition.h"
-
-class glyph;
+#include "glyph.h"
 
 
 class TeXFont {
@@ -25,14 +23,13 @@ class TeXFont {
   
   virtual ~TeXFont();
   
-  void setDisplayResolution(void)
+  void setDisplayResolution()
     {
       for(unsigned int i=0; i<TeXFontDefinition::max_num_of_chars_in_font; i++)
 	glyphtable[i].shrunkenCharacter.resize(0, 0);
     };
   
-  virtual glyph *getGlyph(Q_UINT16 character, bool generateCharacterPixmap=false, QColor color=Qt::black) =0;
-  
+  virtual glyph* getGlyph(Q_UINT16 character, bool generateCharacterPixmap=false, const QColor& color=Qt::black) = 0;
 
   // Checksum of the font. Used e.g. by PK fonts. This field is filled
   // in by the constructor, or set to 0.0, if the font format does not
