@@ -49,6 +49,8 @@
  *					  and Luis Miguel Silveira, MIT RLE.
  */
 
+#include <config.h>
+
 #include <kdebug.h>
 #include <klocale.h>
 #include <math.h>
@@ -114,7 +116,7 @@ TeXFont_PK::~TeXFont_PK()
 }
 
 
-glyph *TeXFont_PK::getGlyph(Q_UINT16 ch, bool generateCharacterPixmap, QColor color)
+glyph* TeXFont_PK::getGlyph(Q_UINT16 ch, bool generateCharacterPixmap, const QColor& color)
 {
 #ifdef DEBUG_PK
   kdDebug(4300) << "TeXFont_PK::getGlyph( ch=" << ch << ", generateCharacterPixmap=" << generateCharacterPixmap << " )" << endl;
@@ -419,7 +421,7 @@ int TeXFont_PK::PK_packed_num(FILE *fp)
 }
 
 
-void TeXFont_PK::PK_skip_specials(void)
+void TeXFont_PK::PK_skip_specials()
 {
 #ifdef DEBUG_PK
   kdDebug(4300) << "TeXFont_PK::PK_skip_specials() called" << endl;
@@ -718,7 +720,7 @@ void TeXFont_PK::read_PK_char(unsigned int ch)
 }
 
 
-void TeXFont_PK::read_PK_index(void)
+void TeXFont_PK::read_PK_index()
 {
 #ifdef DEBUG_PK
   kdDebug(4300) << "TeXFont_PK::read_PK_index() called" << endl;
