@@ -5,7 +5,7 @@
 // (C) 2003 Stefan Kebekus
 // Distributed under the GPL
 
-#include <../config.h>
+#include <config.h>
 #ifdef HAVE_FREETYPE
 
 #include <kdebug.h>
