@@ -1,3 +1,4 @@
+// -*- C++ -*-
 //
 // fontprogress.h
 //
@@ -12,11 +13,6 @@
 
 class KProcIO;
 class KProgress;
-class KPushButton;
-class QVBoxLayout; 
-class QHBoxLayout; 
-class QGridLayout; 
-class QFrame;
 class QLabel;
 
 
@@ -32,21 +28,18 @@ class QLabel;
  *
  *
  **/
-
-
-
 class fontProgressDialog : public KDialogBase
 { 
     Q_OBJECT
 
 public:
-    fontProgressDialog( QString helpIndex, QString label, QString abortTip, QString whatsThis, QString ttip,
+    fontProgressDialog( const QString& helpIndex, const QString& label, const QString& abortTip, const QString& whatsThis, const QString& ttip,
 			QWidget* parent = 0, const QString &name = 0, bool progressbar=true );
     ~fontProgressDialog();
     
     /** The number of steps already done is increased, the text received
 	here is analyzed and presented to the user. */
-    void increaseNumSteps( const QString explanation );
+    void increaseNumSteps(const QString& explanation);
     
     /** Used to initialize the progress bar. If the argument proc is
 	non-zero, the associated process will be killed when the "abort"
@@ -59,7 +52,7 @@ public:
 private slots:
     /** Calling this slot does nothing than to kill the process that is
 	pointed to be procIO, if procIO is not zero.*/
-  void killProcIO(void);
+  void killProcIO();
 
 private:
    QLabel* TextLabel1;
