@@ -7,6 +7,8 @@
 
 // Add header files alphabetically
 
+#include <config.h>
+
 #include <kdebug.h>
 
 #include <kapplication.h>
@@ -36,7 +38,7 @@ optionDialogSpecialWidget::optionDialogSpecialWidget( QWidget* parent,  const ch
   editorDescriptionString += i18n("Click 'Help' to learn how to set up Emacs.");
   
   editorNameString        += "Kate";
-  editorCommandString     += "kate --line %l %f";
+  editorCommandString     += "kate --use --line %l %f";
   editorDescriptionString += i18n("Kate perfectly supports inverse search.");
   
   editorNameString        += "Kile";
@@ -124,7 +126,7 @@ void optionDialogSpecialWidget::slotExtraHelpButton( const QString & )
   kapp->invokeHelp( "inv-search", "kdvi" );
 }
 
-void optionDialogSpecialWidget::apply(void)
+void optionDialogSpecialWidget::apply()
 {
   Prefs::setEditorCommand(EditorCommand);
 }
