@@ -23,6 +23,7 @@
  * SUCH DAMAGE.
  */
 
+#include <config.h>
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -31,7 +32,7 @@
 #include <string.h>
 
 #include "dvi.h"
-#include "dviwin.h"
+#include "dviRenderer.h"
 #include "fontpool.h"
 #include "xdvi.h"
 
@@ -63,10 +64,10 @@ extern void oops(QString message);
  *	The main routine
  */
 
-void TeXFontDefinition::read_VF_index(void)
+void TeXFontDefinition::read_VF_index()
 {
 #ifdef DEBUG_FONTS
-  kdDebug(4300) << "font::read_VF_index(void)" << endl;
+  kdDebug(4300) << "font::read_VF_index()" << endl;
 #endif
   FILE *VF_file = file;
   unsigned char	cmnd;
@@ -79,7 +80,7 @@ void TeXFontDefinition::read_VF_index(void)
 #endif
   // Read preamble.
   fseek(VF_file, (long) one(VF_file), 1);	/* skip comment */
-  long file_checksum = four(VF_file);
+  Q_UINT32 file_checksum = four(VF_file);
 
   if (file_checksum && checksum && file_checksum != checksum)
     kdError(4300) << i18n("Checksum mismatch") << "(dvi = " << checksum << "u, vf = " << file_checksum << 
@@ -93,6 +94,7 @@ void TeXFontDefinition::read_VF_index(void)
     Q_UINT32 checksum  = four(VF_file);
     Q_UINT32 scale     = four(VF_file);
     Q_UINT32 design    = four(VF_file);
+    Q_UNUSED(design);
     Q_UINT16 len       = one(VF_file) + one(VF_file); /* sequence point in the middle */
     char *fontname  = new char[len + 1];
     fread(fontname, sizeof(char), len, VF_file);
