@@ -27,6 +27,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. *
 #include <fcntl.h>
 #include "faxexpand.h"
 #include <qstring.h>
+#include <qfile.h>
 #include <kapplication.h>
 #include <klocale.h>
 #include <kdebug.h>
@@ -109,7 +110,7 @@ notetiff(const char *name)
 
 
     if ((tf = fopen(name, "r")) == NULL) {
-        QString mesg = i18n("Unable to open:\n%1\n").arg(name);
+        QString mesg = i18n("Unable to open:\n%1\n").arg(QFile::decodeName(name));
 	kfaxerror(i18n("Sorry"), mesg);
 	return 0;
     }
@@ -146,7 +147,7 @@ notetiff(const char *name)
 
 	if (fseek(tf, IFDoff, SEEK_SET) < 0) {
 	realbad:
-	  str = i18n("Invalid tiff file:\n%1\n").arg(name);
+	  str = i18n("Invalid tiff file:\n%1\n").arg(QFile::decodeName(name));
 	  kfaxerror(i18n("Sorry"),str);
 	bad:
 	    if (strips)
@@ -248,7 +249,7 @@ notetiff(const char *name)
 	    case 279:		/* StripByteCounts */
 		if (count != nstrips) {
 		  str = i18n("In file %1\nStripsPerImage tag 273=%2,tag279=%3\n")
-			      .arg(name).arg(nstrips).arg(count);
+			      .arg(QFile::decodeName(name)).arg(nstrips).arg(count);
 		  kfaxerror(i18n("Message"),str);
 		  goto realbad;
 		}
@@ -418,7 +419,7 @@ getstrip(struct pagenode *pn, int strip)
 	pn->length = pn->strips[strip].size;
     }
     else {
-      str = i18n("Trying to expand too many strips\n%1%n").arg(pn->pathname);
+      str = i18n("Trying to expand too many strips\n%1%n").arg(QFile::decodeName(pn->pathname));
       kfaxerror(i18n("Warning"),str);
       return NULL;
     }
@@ -447,7 +448,7 @@ getstrip(struct pagenode *pn, int strip)
 	/* handle ghostscript / PC Research fax file */
       if (Data[24] != 1 || Data[25] != 0){
 	str = i18n("Only the first page of the PC Research multipage file\n%1\nwill be shown\n")
-		.arg(pn->pathname);
+		.arg(QFile::decodeName(pn->pathname));
 	kfaxerror(i18n("Message"),str);
       }
 	pn->length -= 64;
@@ -461,7 +462,7 @@ getstrip(struct pagenode *pn, int strip)
 	pn->height = G3count(pn, pn->expander == g32expand);
     if (pn->height == 0) {
 
-      str = i18n("No fax found in file:\n%1\n").arg(pn->pathname);
+      str = i18n("No fax found in file:\n%1\n").arg(QFile::decodeName(pn->pathname));
       kfaxerror(i18n("Sorry"),str);
       errno = 0;
       badfile(pn);
