@@ -1,8 +1,8 @@
 /*
-    $Id: options.h,v 1.12 2002/05/19 21:28:08 cumming Exp $
+    $Id: options.h 465369 2005-09-29 14:33:08Z mueller $
 
-       
-    Copyright (C) 1996 Bernd Johannes Wuebben   
+
+    Copyright (C) 1996 Bernd Johannes Wuebben
                        wuebben@math.cornell.edu
 
     This program is free software; you can redistribute it and/or modify
@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-  
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+
 
 
 */
@@ -29,21 +29,16 @@
 
 #include <qapplication.h>
 #include <qfiledialog.h>
-#include <qframe.h> 
-#include <qbuttongroup.h>
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qgroupbox.h>
-#include <qlabel.h>
-#include <qlineedit.h>
-#include <qlistbox.h>
-#include <qpushbutton.h>
-#include <qradiobutton.h>
-#include <qscrollbar.h>
-#include <qtooltip.h>
 #include <kdialogbase.h>
 #include <qstring.h>
 
+class QGroupBox;
+class QLabel;
+class QButtonGroup;
+class QRadioButton;
+class QCheckBox;
+class QRadioButton;
+
 
 struct optionsinfo {
   int geomauto;
@@ -78,7 +73,7 @@ public slots:
       void g32toggled();
       void g4toggled();
       void g3toggled();
-      
+
 private:
 
     QGroupBox	 *bg;
@@ -97,18 +92,18 @@ private:
     QButtonGroup  *lsbgroup;
     QLabel        *lsblabel;
     QCheckBox *lsb;
-    QButtonGroup *rawgroup;	
+    QButtonGroup *rawgroup;
     QRadioButton *g3;
     QRadioButton *g32d;
     QRadioButton *g4;
 
     QLabel 	*rawlabel;
-    
+
     QLabel 	*widthlabel;
     QLabel 	*heightlabel;
     KIntNumInput *widthedit;
     KIntNumInput *heightedit;
-        
+
     struct optionsinfo oi;
 
 };
