@@ -240,7 +240,7 @@ int viewfaxmain()
     firstpage = lastpage = thispage = helppage = auxpage =  0;
 
     for (int i = 0; i < args->count(); i++){
-	loadfile(args->arg(i));
+	loadfile(QFile::decodeName(args->arg(i)));
     }
     args->clear();
 
