@@ -1,19 +1,19 @@
 /* This file is part of the KDE project
  * Copyright (C) 2002 Shane Wright <me@shanewright.co.uk>
  *
- * This program is free softxbmre; you can redistribute it and/or
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
- * License as published by the Free Softxbmre Foundation version 2.
+ * License as published by the Free Software Foundation version 2.
  *
  * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied xbmrranty of
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; see the file COPYING.  If not, write to
- * the Free Softxbmre Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -108,11 +108,11 @@ bool KXbmPlugin::readInfo( KFileMetaInfo& info, uint what)
     char linebuf[1000];
 
     // read the first line
-    file.readLine(linebuf, 1000);
+    file.readLine(linebuf, sizeof( linebuf ));
     uint32_t width = xbm_processLine(linebuf);
 
     // read the 2nd line
-    file.readLine(linebuf, 1000);
+    file.readLine(linebuf, sizeof( linebuf ));
     uint32_t height = xbm_processLine(linebuf);
 
     if ((width > 0) && (height > 0)) {
