@@ -118,6 +118,7 @@ KGamma::~KGamma() {
       }
     delete rootProcess;
   }
+  delete xv;
 }
 
 /** User interface */
@@ -283,10 +284,17 @@ void KGamma::setupUI() {
   }
 }
 
-/** Restore latest saved gamma values */
 void KGamma::load() {
+	load( false );
+}
+
+/** Restore latest saved gamma values */
+void KGamma::load(bool useDefaults) {
   if (GammaCorrection) {
     KConfig *config = new KConfig("kgammarc");
+
+	 config->setReadDefaults( useDefaults );
+
     config->setGroup("ConfigFile");
 
     // save checkbox status
@@ -322,7 +330,7 @@ void KGamma::load() {
     }
     xv->setScreen(currentScreen);
 
-    emit changed(false);
+    emit changed(useDefaults);
   }
 }
 
@@ -373,16 +381,7 @@ void KGamma::save() {
 }
 
 void KGamma::defaults() {
-  if (GammaCorrection) {
-    for (int i = 0; i < ScreenCount; i++) {
-      xv->setScreen(i);
-      gctrl->setGamma("1.00");
-    }
-    xv->setScreen(currentScreen);
-
-  }
-  xf86cfgbox->setChecked(false);
-  syncbox->setChecked(false);
+	load( true );
 }
 
 bool KGamma::loadSettings() {
