@@ -17,7 +17,7 @@
   copies.
 */
 
-/* $Id: dscparse.cpp 420737 2005-06-01 09:09:35Z wgreven $ */
+/* $Id: dscparse.cpp 636821 2007-02-24 10:47:12Z mueller $ */
 
 /* dscparse.c - DSC parser  */
 
@@ -82,6 +82,9 @@
 /* If document is DOS EPS and we haven't read 30 bytes, ask for more. */
 #define CDSC_NEEDMORE 11
 
+template<class T>
+inline const T min (const T& a, const T& b) { return a < b ? a : b; }
+
 /* local prototypes */
 dsc_private void * dsc_memalloc(P2(CDSC *dsc, size_t size));
 dsc_private void dsc_memfree(P2(CDSC*dsc, void *ptr));
@@ -1090,7 +1093,7 @@ dsc_read_line(CDSC *dsc)
 dsc_private void 
 dsc_save_line(CDSC *dsc)
 {
-    int len = min(sizeof(dsc->last_line), dsc->line_length);
+    int len = min((size_t) sizeof(dsc->last_line), (size_t) dsc->line_length);
     memcpy(dsc->last_line, dsc->line, len);
 }
 
@@ -1100,7 +1103,7 @@ dsc_unknown(CDSC *dsc)
 {
     if (dsc->debug_print_fn) {
 	char line[DSC_LINE_LENGTH];
-	unsigned int length = min(DSC_LINE_LENGTH-1, dsc->line_length);
+	unsigned int length = min((unsigned)DSC_LINE_LENGTH-1, dsc->line_length);
 	sprintf(line, "Unknown in %s section at line %d:\n  ", 
 	    dsc_scan_section_name[dsc->scan_section], dsc->line_count);
 	dsc_debug_print(dsc, line);
@@ -1643,7 +1646,7 @@ dsc_parse_media(CDSC *dsc, const CDSCMEDIA **page_media)
     int n = IS_DSC(dsc->line, "%%+") ? 3 : 12; /* %%PageMedia: */
     unsigned int i;
 
-    if (dsc_copy_string(media_name, sizeof(media_name)-1, 
+    if (dsc_copy_string(media_name, sizeof(media_name)-1,
 	dsc->line+n, dsc->line_length-n, NULL)) {
 	for (i=0; i<dsc->media_count; i++) {
 	    if (dsc->media[i]->name && 
@@ -1689,7 +1692,7 @@ dsc_parse_document_media(CDSC *dsc)
 	lmedia.name = lmedia.colour = lmedia.type = (char *)NULL;
 	lmedia.width = lmedia.height = lmedia.weight = 0;
 	lmedia.mediabox = (CDSCBBOX *)NULL;
-	lmedia.name = dsc_copy_string(name, sizeof(name),
+	lmedia.name = dsc_copy_string(name, sizeof(name)-1,
 		dsc->line+n, dsc->line_length-n, &i);
 	n+=i;
 	if (i)
@@ -1702,11 +1705,11 @@ dsc_parse_document_media(CDSC *dsc)
 	    lmedia.weight = dsc_get_real(dsc->line+n, dsc->line_length-n, &i);
 	n+=i;
 	if (i)
-	    lmedia.colour = dsc_copy_string(colour, sizeof(colour),
+	    lmedia.colour = dsc_copy_string(colour, sizeof(colour)-1,
 		dsc->line+n, dsc->line_length-n, &i);
 	n+=i;
 	if (i)
-	    lmedia.type = dsc_copy_string(type, sizeof(type),
+	    lmedia.type = dsc_copy_string(type, sizeof(type)-1,
 		dsc->line+n, dsc->line_length-n, &i);
 
 	if (i==0)
@@ -3283,7 +3286,7 @@ dsc_get_int(const char *line, unsigned int len, unsigned int *offset)
     unsigned int i = 0;
     unsigned char ch;
 
-    len = min(len, sizeof(newline)-1);
+    len = min((size_t) len, (size_t) sizeof(newline)-1);
     while ((i<len) && IS_WHITE(line[i]))
 	i++;	/* skip leading spaces */
     while (i < len) {
@@ -3309,7 +3312,7 @@ dsc_get_real(const char *line, unsigned int len, unsigned int *offset)
     unsigned int i = 0;
     unsigned char ch;
 
-    len = min(len, sizeof(newline)-1);
+    len = min((size_t) len, (size_t) sizeof(newline)-1);
     while ((i<len) && IS_WHITE(line[i]))
 	i++;	/* skip leading spaces */
     while (i < len) {
@@ -3354,7 +3357,7 @@ dsc_parse_page(CDSC *dsc)
     int page_number;
 
     p = dsc->line + 7;
-    pl = dsc_copy_string(page_label, sizeof(page_label), p, dsc->line_length-7, &i);
+    pl = dsc_copy_string(page_label, sizeof(page_label)-1, p, dsc->line_length-7, &i);
     if (pl == NULL)
 	return CDSC_ERROR;
     p += i;
