@@ -205,11 +205,15 @@ void KGVMiniWidget::firstPage()
 
 void KGVMiniWidget::lastPage()
 {
+    if ( !dsc() ) return;
+
     goToPage( dsc()->page_count() - 1 );
 }
 
 bool KGVMiniWidget::prevPage()
 {
+    if ( !dsc() ) return false;
+
     int new_page = 0;
 
     if( dsc()->isStructured() ) {
@@ -224,6 +228,8 @@ bool KGVMiniWidget::prevPage()
 
 bool KGVMiniWidget::nextPage()
 {
+    if ( !dsc() ) return false;
+
     int new_page = 0;
 
     if( dsc()->isStructured() ) {
