@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qcheckbox.h>
@@ -246,7 +246,7 @@ bool MrmlPart::openURL( const KURL& url )
         if ( !list.isEmpty() )
             KMessageBox::sorry( m_view,
                                 i18n("You can only search by example images "
-                                     "on a local Indexing Server"),
+                                     "on a local indexing server."),
                                 i18n("Only Local Servers Possible") );
     }
 
@@ -279,8 +279,8 @@ bool MrmlPart::openURL( const KURL& url )
                                   "specified. Do you want to configure them "
                                   "now?"),
                                             i18n("Configuration Missing"),
-                                            KStdGuiItem::yes(),
-                                            KStdGuiItem::no(),
+                                            i18n("Configure"),
+                                            i18n("Do Not Configure"),
                                             "kmrml_ask_configure_gift" )
                  == KMessageBox::Yes )
             {
@@ -526,8 +526,8 @@ void MrmlPart::parseQueryResult( QDomElement& queryResult )
                 for ( ; it != list.end(); ++it )
                 {
                     QDomNamedNodeMap a = (*it).attributes();
-                    m_view->addItem( (*it).attribute("image-location" ),
-                                     (*it).attribute("thumbnail-location" ),
+                    m_view->addItem( KURL( (*it).attribute("image-location" ) ),
+                                     KURL( (*it).attribute("thumbnail-location" ) ),
                                      (*it).attribute("calculated-similarity"));
 
                 }
@@ -718,7 +718,7 @@ void MrmlPart::slotActivated( const KURL& url, ButtonState button )
 
 void MrmlPart::enableExtensionActions( const KURL& url, bool enable )
 {
-    bool del = KProtocolInfo::supportsDeleting( url.protocol() );
+    bool del = KProtocolInfo::supportsDeleting( url );
     emit m_browser->enableAction( "copy", enable );
     emit m_browser->enableAction( "trash", del );
     emit m_browser->enableAction( "del",   del );
