@@ -65,8 +65,14 @@ int main (int argc, char *argv [])
         I18N_NOOP ("Paint Program for KDE"),
         KAboutData::License_Custom,
         0/*copyright statement - see licence instead*/,
+#if 1
+        0/*free text*/,
+// STRING: After string freeze, make below the "free text" but point to
+//         http://www.kolourpaint.org/support.
+#else
         I18N_NOOP ("Support / Feedback:\n"
                    "kolourpaint-support@lists.sourceforge.net\n"),
+#endif
         "http://www.kolourpaint.org/"
     );
 
@@ -75,6 +81,9 @@ int main (int argc, char *argv [])
     aboutData.setLicenseText (kpLicenseText);
 
 
+// STRING: After string freeze, make below the custom author text but point to
+//         http://www.kolourpaint.org/support.
+#if 0
     aboutData.setCustomAuthorText (
         I18N_NOOP
         (
@@ -95,6 +104,7 @@ int main (int argc, char *argv [])
             "<br>"
             "</qt>"
         ));
+#endif
 
 
     // SYNC: with AUTHORS
@@ -106,6 +116,9 @@ int main (int argc, char *argv [])
     aboutData.addAuthor ("Kazuki Ohta", I18N_NOOP ("InputMethod Support"), "mover@hct.zaq.ne.jp");
     aboutData.addAuthor ("Nuno Pinheiro", I18N_NOOP ("Icons"), "nf.pinheiro@gmail.com");
     aboutData.addAuthor ("Danny Allen", I18N_NOOP ("Icons"), "dannya40uk@yahoo.co.uk");
+    aboutData.addAuthor ("Martin Koller",
+                         0/*STRING: string freeze prevents us from writing: Scanning Support*/,
+                         "m.koller@surfeu.at");
 
 
     aboutData.addCredit ("Rashid N. Achilov");
@@ -217,7 +230,11 @@ int main (int argc, char *argv [])
 
 
     if (app.isRestored ())
+    {
+        // Creates a kpMainWindow using the default constructor and then
+        // calls kpMainWindow::readProperties().
         RESTORE (kpMainWindow)
+    }
     else
     {
         kpMainWindow *mainWindow;
