@@ -52,6 +52,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kmimetype.h>
+#include <ksavefile.h>
 #include <ktempfile.h>
 
 #include <kpcolor.h>
@@ -125,69 +126,16 @@ void kpDocument::setMainWindow (kpMainWindow *mainWindow)
  */
 
 // public static
-QPixmap kpDocument::getPixmapFromFile (const KURL &url, bool suppressDoesntExistDialog,
-                                       QWidget *parent,
-                                       kpDocumentSaveOptions *saveOptions,
-                                       kpDocumentMetaInfo *metaInfo)
+QPixmap kpDocument::convertToPixmapAsLosslessAsPossible (
+        const QImage &image,
+        const kpPixmapFX::WarnAboutLossInfo &wali,
+
+        kpDocumentSaveOptions *saveOptions,
+        kpDocumentMetaInfo *metaInfo)
 {
-#if DEBUG_KP_DOCUMENT
-    kdDebug () << "kpDocument::getPixmapFromFile(" << url << "," << parent << ")" << endl;
-#endif
-
-    if (saveOptions)
-        *saveOptions = kpDocumentSaveOptions ();
-
-    if (metaInfo)
-        *metaInfo = kpDocumentMetaInfo ();
-
-
-    QString tempFile;
-    if (url.isEmpty () || !KIO::NetAccess::download (url, tempFile, parent))
-    {
-        if (!suppressDoesntExistDialog)
-        {
-            KMessageBox::sorry (parent,
-                                i18n ("Could not open \"%1\".")
-                                    .arg (kpDocument::prettyFilenameForURL (url)));
-        }
-
-        return QPixmap ();
-    }
-
-
-    // sync: remember to "KIO::NetAccess::removeTempFile (tempFile)" in all exit paths
-
-    QString detectedMimeType = KImageIO::mimeType (tempFile);
-    if (saveOptions)
-        saveOptions->setMimeType (detectedMimeType);
-
-#if DEBUG_KP_DOCUMENT
-    kdDebug () << "\ttempFile=" << tempFile << endl;
-    kdDebug () << "\tmimetype=" << detectedMimeType << endl;
-    kdDebug () << "\tsrc=" << url.path () << endl;
-    kdDebug () << "\tmimetype of src=" << KImageIO::mimeType (url.path ()) << endl;
-#endif
-
-    if (detectedMimeType.isEmpty ())
-    {
-        KMessageBox::sorry (parent,
-                            i18n ("Could not open \"%1\" - unknown mimetype.")
-                                .arg (kpDocument::prettyFilenameForURL (url)));
-        KIO::NetAccess::removeTempFile (tempFile);
-        return QPixmap ();
-    }
-
-
-    QImage image (tempFile);
     if (image.isNull ())
-    {
-        KMessageBox::sorry (parent,
-                            i18n ("Could not open \"%1\" - unsupported image format.\n"
-                                  "The file may be corrupt.")
-                                .arg (kpDocument::prettyFilenameForURL (url)));
-        KIO::NetAccess::removeTempFile (tempFile);
         return QPixmap ();
-    }
+
 
 #if DEBUG_KP_DOCUMENT
     kdDebug () << "\timage: depth=" << image.depth ()
@@ -241,31 +189,7 @@ QPixmap kpDocument::getPixmapFromFile (const KURL &url, bool suppressDoesntExist
 #endif
 
 
-    QPixmap newPixmap = kpPixmapFX::convertToPixmapAsLosslessAsPossible (image,
-        kpPixmapFX::WarnAboutLossInfo (
-             i18n ("The image \"%1\""
-                   " may have more colors than the current screen mode."
-                   " In order to display it, some colors may be changed."
-                   " Try increasing your screen depth to at least %2bpp."
-
-                   "\nIt also"
-
-                   " contains translucency which is not fully"
-                   " supported. The translucency data will be"
-                   " approximated with a 1-bit transparency mask.")
-                 .arg (prettyFilenameForURL (url)),
-             i18n ("The image \"%1\""
-                   " may have more colors than the current screen mode."
-                   " In order to display it, some colors may be changed."
-                   " Try increasing your screen depth to at least %2bpp.")
-                 .arg (prettyFilenameForURL (url)),
-             i18n ("The image \"%1\""
-                   " contains translucency which is not fully"
-                   " supported. The translucency data will be"
-                   " approximated with a 1-bit transparency mask.")
-                .arg (prettyFilenameForURL (url)),
-            "docOpen",
-            parent));
+    QPixmap newPixmap = kpPixmapFX::convertToPixmapAsLosslessAsPossible (image, wali);
 
 
 #if DEBUG_KP_DOCUMENT && 1
@@ -302,12 +226,112 @@ QPixmap kpDocument::getPixmapFromFile (const KURL &url, bool suppressDoesntExist
 }
 #endif
 
+    return newPixmap;
+}
+
+// public static
+QPixmap kpDocument::getPixmapFromFile (const KURL &url, bool suppressDoesntExistDialog,
+                                       QWidget *parent,
+                                       kpDocumentSaveOptions *saveOptions,
+                                       kpDocumentMetaInfo *metaInfo)
+{
+#if DEBUG_KP_DOCUMENT
+    kdDebug () << "kpDocument::getPixmapFromFile(" << url << "," << parent << ")" << endl;
+#endif
+
+    if (saveOptions)
+        *saveOptions = kpDocumentSaveOptions ();
+
+    if (metaInfo)
+        *metaInfo = kpDocumentMetaInfo ();
+
+
+    QString tempFile;
+    if (url.isEmpty () || !KIO::NetAccess::download (url, tempFile, parent))
+    {
+        if (!suppressDoesntExistDialog)
+        {
+            KMessageBox::sorry (parent,
+                                i18n ("Could not open \"%1\".")
+                                    .arg (kpDocument::prettyFilenameForURL (url)));
+        }
+
+        return QPixmap ();
+    }
+
+
+    QImage image;
+
+    // sync: remember to "KIO::NetAccess::removeTempFile (tempFile)" in all exit paths
+    {
+        QString detectedMimeType = KImageIO::mimeType (tempFile);
+        if (saveOptions)
+            saveOptions->setMimeType (detectedMimeType);
+
+    #if DEBUG_KP_DOCUMENT
+        kdDebug () << "\ttempFile=" << tempFile << endl;
+        kdDebug () << "\tmimetype=" << detectedMimeType << endl;
+        kdDebug () << "\tsrc=" << url.path () << endl;
+        kdDebug () << "\tmimetype of src=" << KImageIO::mimeType (url.path ()) << endl;
+    #endif
+
+        if (detectedMimeType.isEmpty ())
+        {
+            KMessageBox::sorry (parent,
+                                i18n ("Could not open \"%1\" - unknown mimetype.")
+                                    .arg (kpDocument::prettyFilenameForURL (url)));
+            KIO::NetAccess::removeTempFile (tempFile);
+            return QPixmap ();
+        }
+
+
+        image = QImage (tempFile);
+        KIO::NetAccess::removeTempFile (tempFile);
+    }
+
+
+    if (image.isNull ())
+    {
+        KMessageBox::sorry (parent,
+                            i18n ("Could not open \"%1\" - unsupported image format.\n"
+                                  "The file may be corrupt.")
+                                .arg (kpDocument::prettyFilenameForURL (url)));
+        return QPixmap ();
+    }
+
+    const QPixmap newPixmap = kpDocument::convertToPixmapAsLosslessAsPossible (image,
+        kpPixmapFX::WarnAboutLossInfo (
+             i18n ("The image \"%1\""
+                   " may have more colors than the current screen mode."
+                   " In order to display it, some colors may be changed."
+                   " Try increasing your screen depth to at least %2bpp."
+
+                   "\nIt also"
+
+                   " contains translucency which is not fully"
+                   " supported. The translucency data will be"
+                   " approximated with a 1-bit transparency mask.")
+                 .arg (prettyFilenameForURL (url)),
+             i18n ("The image \"%1\""
+                   " may have more colors than the current screen mode."
+                   " In order to display it, some colors may be changed."
+                   " Try increasing your screen depth to at least %2bpp.")
+                 .arg (prettyFilenameForURL (url)),
+             i18n ("The image \"%1\""
+                   " contains translucency which is not fully"
+                   " supported. The translucency data will be"
+                   " approximated with a 1-bit transparency mask.")
+                .arg (prettyFilenameForURL (url)),
+            "docOpen",
+            parent),
+        saveOptions,
+        metaInfo);
+
     if (newPixmap.isNull ())
     {
         KMessageBox::sorry (parent,
                             i18n ("Could not open \"%1\" - out of graphics memory.")
                                 .arg (kpDocument::prettyFilenameForURL (url)));
-        KIO::NetAccess::removeTempFile (tempFile);
         return QPixmap ();
     }
 
@@ -319,7 +343,6 @@ QPixmap kpDocument::getPixmapFromFile (const KURL &url, bool suppressDoesntExist
 #endif
 
 
-    KIO::NetAccess::removeTempFile (tempFile);
     return newPixmap;
 }
 
@@ -461,7 +484,7 @@ bool kpDocument::lossyPromptContinue (const QPixmap &pixmap,
                 i18n ("<qt><p>Saving the image at the low color depth of %1-bit"
                         " may result in the loss of color information."
 
-                        " Any transparency will also be removed."
+                        " Any transparency will also be removed.</p>"
 
                         "<p>Are you sure you want to save at this color depth?</p></qt>")
                     .arg (saveOptions.colorDepth ()),
@@ -573,6 +596,20 @@ bool kpDocument::savePixmapToDevice (const QPixmap &pixmap,
     return true;
 }
 
+static void CouldNotCreateTemporaryFileDialog (QWidget *parent)
+{
+    KMessageBox::error (parent,
+                        i18n ("Could not save image - unable to create temporary file."));
+}
+
+static void CouldNotSaveDialog (const KURL &url, QWidget *parent)
+{
+    // TODO: use file.errorString()
+    KMessageBox::error (parent,
+                        i18n ("Could not save as \"%1\".")
+                            .arg (kpDocument::prettyFilenameForURL (url)));
+}
+
 // public static
 bool kpDocument::savePixmapToFile (const QPixmap &pixmap,
                                    const KURL &url,
@@ -621,62 +658,118 @@ bool kpDocument::savePixmapToFile (const QPixmap &pixmap,
     }
 
 
-    KTempFile tempFile;
-    tempFile.setAutoDelete (true);
-
-    QString filename;
-
-    if (!url.isLocalFile ())
+    // Local file?
+    if (url.isLocalFile ())
     {
-        filename = tempFile.name ();
+        const QString filename = url.path ();
+
+        // sync: All failure exit paths _must_ call KSaveFile::abort() or
+        //       else, the KSaveFile destructor will overwrite the file,
+        //       <filename>, despite the failure.
+        KSaveFile atomicFileWriter (filename);
+        {
+            if (atomicFileWriter.status () != 0)
+            {
+                // We probably don't need this as <filename> has not been
+                // opened.
+                atomicFileWriter.abort ();
+
+            #if DEBUG_KP_DOCUMENT
+                kdDebug () << "\treturning false because could not open KSaveFile"
+                           << " status=" << atomicFileWriter.status () << endl;
+            #endif
+                ::CouldNotCreateTemporaryFileDialog (parent);
+                return false;
+            }
+
+            // Write to local temporary file.
+            if (!savePixmapToDevice (pixmap, atomicFileWriter.file (),
+                                     saveOptions, metaInfo,
+                                     false/*no lossy prompt*/,
+                                     parent))
+            {
+                atomicFileWriter.abort ();
+
+            #if DEBUG_KP_DOCUMENT
+                kdDebug () << "\treturning false because could not save pixmap to device"
+                            << endl;
+            #endif
+                ::CouldNotSaveDialog (url, parent);
+                return false;
+            }
+
+            // Atomically overwrite local file with the temporary file
+            // we saved to.
+            if (!atomicFileWriter.close ())
+            {
+                atomicFileWriter.abort ();
+
+            #if DEBUG_KP_DOCUMENT
+                kdDebug () << "\tcould not close KSaveFile" << endl;
+            #endif
+                ::CouldNotSaveDialog (url, parent);
+                return false;
+            }
+        }  // sync KSaveFile.abort()
+    }
+    // Remote file?
+    else
+    {
+        // Create temporary file that is deleted when the variable goes
+        // out of scope.
+        KTempFile tempFile;
+        tempFile.setAutoDelete (true);
+
+        QString filename = tempFile.name ();
         if (filename.isEmpty ())
         {
         #if DEBUG_KP_DOCUMENT
             kdDebug () << "\treturning false because tempFile empty" << endl;
         #endif
-            KMessageBox::error (parent,
-                                i18n ("Could not save image - unable to create temporary file."));
+            ::CouldNotCreateTemporaryFileDialog (parent);
             return false;
         }
-    }
-    else
-        filename = url.path ();
 
-
-    QFile file (filename);
-    bool fileOpenOK = false;
-    if (!(fileOpenOK = file.open (IO_WriteOnly)) ||
-        !savePixmapToDevice (pixmap, &file,
-                             saveOptions, metaInfo,
-                             false/*no lossy prompt*/,
-                             parent) ||
-        (file.close (),
-         file.status () != IO_Ok))
-    {
-    #if DEBUG_KP_DOCUMENT
-        if (!fileOpenOK)
+        // Write to local temporary file.
+        QFile file (filename);
         {
-            kdDebug () << "\treturning false because fileOpenOK=false"
-                       << " errorString=" << file.errorString () << endl;
+            if (!file.open (IO_WriteOnly))
+            {
+            #if DEBUG_KP_DOCUMENT
+                kdDebug () << "\treturning false because can't open file"
+                            << " errorString=" << file.errorString () << endl;
+            #endif
+                ::CouldNotCreateTemporaryFileDialog (parent);
+                return false;
+            }
+
+            if (!savePixmapToDevice (pixmap, &file,
+                                     saveOptions, metaInfo,
+                                     false/*no lossy prompt*/,
+                                     parent))
+            {
+            #if DEBUG_KP_DOCUMENT
+                kdDebug () << "\treturning false because could not save pixmap to device"
+                            << endl;
+            #endif
+                ::CouldNotSaveDialog (url, parent);
+                return false;
+            }
         }
-        else
+        file.close ();
+        if (file.status () != IO_Ok)
         {
-            kdDebug () << "\treturning false because could not save pixmap to device"
-                       << endl;
+        #if DEBUG_KP_DOCUMENT
+            kdDebug () << "\treturning false because could not close" << endl;
+        #endif
+            ::CouldNotSaveDialog (url, parent);
+            return false;
         }
-    #endif
 
-        // TODO: use file.errorString()
-        KMessageBox::error (parent,
-                            i18n ("Could not save as \"%1\".")
-                                .arg (kpDocument::prettyFilenameForURL (url)));
-
-        return false;
-    }
-
-
-    if (!url.isLocalFile ())
-    {
+        // Copy local temporary file to overwrite remote.
+        // TODO: No one seems to know how to do this atomically
+        //       [http://lists.kde.org/?l=kde-core-devel&m=117845162728484&w=2].
+        //       At least, fish:// (ssh) is definitely not atomic.
         if (!KIO::NetAccess::upload (filename, url, parent))
         {
         #if DEBUG_KP_DOCUMENT
