@@ -60,6 +60,7 @@ class KToggleAction;
 class KToolBar;
 class KPrinter;
 class KRecentFilesAction;
+class KScanDialog;
 class KToggleFullScreenAction;
 
 class kpColor;
@@ -119,6 +120,15 @@ private:
     void readThumbnailSettings ();
     void init ();
 
+    // (only called for restoring a previous session e.g. starting KDE with
+    //  a previously saved session; it's not called on normal KolourPaint
+    //  startup)
+    virtual void readProperties (KConfig *cfg);
+    // (only called for saving the current session e.g. logging out of KDE
+    //  with the KolourPaint window open; it's not called on normal KolourPaint
+    //  exit)
+    virtual void saveProperties (KConfig *cfg);
+
 public:
     ~kpMainWindow ();
 
@@ -280,21 +290,21 @@ private:
 
     KAction *m_actionNew, *m_actionOpen;
     KRecentFilesAction *m_actionOpenRecent;
-    KAction *m_actionSave, *m_actionSaveAs, *m_actionExport,
+    KAction *m_actionScan, *m_actionSave, *m_actionSaveAs, *m_actionExport,
             *m_actionReload,
             *m_actionPrint, *m_actionPrintPreview,
             *m_actionMail,
             *m_actionSetAsWallpaperTiled, *m_actionSetAsWallpaperCentered,
             *m_actionClose, *m_actionQuit;
 
+    KScanDialog *m_scanDialog;
+
     KURL m_lastExportURL;
     kpDocumentSaveOptions m_lastExportSaveOptions;
     bool m_exportFirstTime;
 
 private:
-    bool shouldOpenInNewWindow () const;
     void addRecentURL (const KURL &url);
-    void setRecentURLs (const QStringList &items);
 
 private slots:
     void slotNew ();
@@ -304,7 +314,26 @@ private:
     void saveDefaultDocSize (const QSize &size);
 
 private:
+    bool shouldOpenInNewWindow () const;
+    void setDocumentChoosingWindow (kpDocument *doc);
+
+private:
+    kpDocument *openInternal (const KURL &url,
+        const QSize &fallbackDocSize,
+        bool newDocSameNameIfNotExist);
+    // Same as above except that it:
+    //
+    // 1. Assumes a default fallback document size.
+    // 2. If the URL is successfully opened (with the special exception of
+    //    the "kolourpaint doesnotexist.png" case), it is bubbled up to the
+    //    top in the Recent Files Action.
+    //
+    // As a result of this behavior, this should only be called in response
+    // to a user open request e.g. File / Open or "kolourpaint doesexist.png".
+    // It should not be used for session restore - in that case, it does not
+    // make sense to bubble the Recent Files list.
     bool open (const KURL &url, bool newDocSameNameIfNotExist = false);
+
     KURL::List askForOpenURLs (const QString &caption,
                                const QString &startURL,
                                bool allowMultipleURLs = true);
@@ -313,6 +342,9 @@ private slots:
     void slotOpen ();
     void slotOpenRecent (const KURL &url);
 
+    void slotScan ();
+    void slotScanned (const QImage &image, int);
+
     bool save (bool localOnly = false);
     bool slotSave ();
 
@@ -340,7 +372,7 @@ private slots:
 
 private:
     void sendFilenameToPrinter (KPrinter *printer);
-    void sendPixmapToPrinter (KPrinter *printer);
+    void sendPixmapToPrinter (KPrinter *printer, bool showPrinterSetupDialog);
 
 private slots:
     void slotPrint ();
@@ -462,14 +494,14 @@ private slots:
 public:
     void zoomIn (bool centerUnderCursor = false);
     void zoomOut (bool centerUnderCursor = false);
-    
+
 public slots:
     void slotZoomIn ();
     void slotZoomOut ();
 
 private:
     void zoomAccordingToZoomAction (bool centerUnderCursor = false);
-    
+
 private slots:
     void slotZoom ();
 
