@@ -30,6 +30,7 @@
 
 #include <qcstring.h>
 #include <qdatastream.h>
+#include <qpaintdevicemetrics.h>
 #include <qpainter.h>
 #include <qsize.h>
 
@@ -39,6 +40,7 @@
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kfiledialog.h>
+#include <kiconloader.h>
 #include <kimagefilepreview.h>
 #include <kimageio.h>
 #include <kio/netaccess.h>
@@ -47,6 +49,7 @@
 #include <kprinter.h>
 #include <kstdaccel.h>
 #include <kstdaction.h>
+#include <kscan.h>
 
 #include <kpdefs.h>
 #include <kpdocument.h>
@@ -59,6 +62,9 @@
 // private
 void kpMainWindow::setupFileMenuActions ()
 {
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpMainWindow::setupFileMenuActions()" << endl;
+#endif
     KActionCollection *ac = actionCollection ();
 
     m_actionNew = KStdAction::openNew (this, SLOT (slotNew ()), ac);
@@ -66,6 +72,9 @@ void kpMainWindow::setupFileMenuActions ()
 
     m_actionOpenRecent = KStdAction::openRecent (this, SLOT (slotOpenRecent (const KURL &)), ac);
     m_actionOpenRecent->loadEntries (kapp->config ());
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\trecent URLs=" << m_actionOpenRecent->items () << endl;
+#endif
 
     m_actionSave = KStdAction::save (this, SLOT (slotSave ()), ac);
     m_actionSaveAs = KStdAction::saveAs (this, SLOT (slotSaveAs ()), ac);
@@ -73,6 +82,9 @@ void kpMainWindow::setupFileMenuActions ()
     m_actionExport = new KAction (i18n ("E&xport..."), 0,
         this, SLOT (slotExport ()), ac, "file_export");
 
+    m_actionScan = new KAction (i18n ("Scan..."), SmallIcon ("scanner"), 0,
+        this, SLOT (slotScan ()), ac, "file_scan");
+
     //m_actionRevert = KStdAction::revert (this, SLOT (slotRevert ()), ac);
     m_actionReload = new KAction (i18n ("Reloa&d"), KStdAccel::reload (),
         this, SLOT (slotReload ()), ac, "file_revert");
@@ -91,6 +103,8 @@ void kpMainWindow::setupFileMenuActions ()
     m_actionClose = KStdAction::close (this, SLOT (slotClose ()), ac);
     m_actionQuit = KStdAction::quit (this, SLOT (slotQuit ()), ac);
 
+    m_scanDialog = 0;
+
     enableFileMenuDocumentActions (false);
 }
 
@@ -122,12 +136,6 @@ void kpMainWindow::enableFileMenuDocumentActions (bool enable)
 }
 
 
-// private
-bool kpMainWindow::shouldOpenInNewWindow () const
-{
-    return (m_document && !m_document->isEmpty ());
-}
-
 // private
 void kpMainWindow::addRecentURL (const KURL &url)
 {
@@ -181,22 +189,25 @@ void kpMainWindow::addRecentURL (const KURL &url)
         #endif
 
             if (mw != this)
-                mw->setRecentURLs (m_actionOpenRecent->items ());
+            {
+                // WARNING: Do not use KRecentFilesAction::setItems()
+                //          - it does not work since only its superclass,
+                //          KSelectAction, implements setItems() and can't
+                //          update KRecentFilesAction's URL list.
+
+                // Avoid URL memory leak in KRecentFilesAction::loadEntries().
+                mw->m_actionOpenRecent->clearURLList ();
+
+                mw->m_actionOpenRecent->loadEntries (cfg);
+            #if DEBUG_KP_MAIN_WINDOW
+                kdDebug () << "\t\t\tcheck recent URLs="
+                           << mw->m_actionOpenRecent->items () << endl;
+            #endif
+            }
         }
     }
 }
 
-// private
-void kpMainWindow::setRecentURLs (const QStringList &items)
-{
-#if DEBUG_KP_MAIN_WINDOW
-    kdDebug () << "kpMainWindow(" << name () << ")::setRecentURLs()" << endl;
-    kdDebug () << "\titems=" << items << endl;
-#endif
-    m_actionOpenRecent->setItems (items);
-}
-
-
 
 // private slot
 void kpMainWindow::slotNew ()
@@ -247,9 +258,7 @@ QSize kpMainWindow::defaultDocSize () const
 void kpMainWindow::saveDefaultDocSize (const QSize &size)
 {
 #if DEBUG_KP_MAIN_WINDOW
-    kdDebug () << "\tCONFIG: saving Last Doc Size = "
-               << QSize (dialog.imageWidth (), dialog.imageHeight ())
-               << endl;
+    kdDebug () << "\tCONFIG: saving Last Doc Size = " << size << endl;
 #endif
 
     KConfigGroupSaver cfgGroupSaver (kapp->config (), kpSettingsGroupGeneral);
@@ -261,39 +270,69 @@ void kpMainWindow::saveDefaultDocSize (const QSize &size)
 
 
 // private
-bool kpMainWindow::open (const KURL &url, bool newDocSameNameIfNotExist)
+bool kpMainWindow::shouldOpenInNewWindow () const
 {
-    QSize docSize = defaultDocSize ();
-
-    // create doc
-    kpDocument *newDoc = new kpDocument (docSize.width (), docSize.height (), this);
-    if (newDoc->open (url, newDocSameNameIfNotExist))
-    {
-        if (newDoc->isFromURL (false/*don't bother checking exists*/))
-            addRecentURL (url);
-    }
-    else
-    {
-        delete newDoc;
-        return false;
-    }
+    return (m_document && !m_document->isEmpty ());
+}
 
+// private
+void kpMainWindow::setDocumentChoosingWindow (kpDocument *doc)
+{
     // need new window?
     if (shouldOpenInNewWindow ())
     {
         // send doc to new window
-        kpMainWindow *win = new kpMainWindow (newDoc);
+        kpMainWindow *win = new kpMainWindow (doc);
         win->show ();
     }
     else
     {
         // set up views, doc signals
-        setDocument (newDoc);
+        setDocument (doc);
+    }
+}
+
+
+// private
+kpDocument *kpMainWindow::openInternal (const KURL &url,
+        const QSize &fallbackDocSize,
+        bool newDocSameNameIfNotExist)
+{
+    // create doc
+    kpDocument *newDoc = new kpDocument (fallbackDocSize.width (),
+                                         fallbackDocSize.height (),
+                                         this);
+    if (!newDoc->open (url, newDocSameNameIfNotExist))
+    {
+        delete newDoc;
+        return 0;
     }
 
-    return true;
+    // Send document to current or new window.
+    setDocumentChoosingWindow (newDoc);
+
+    return newDoc;
 }
 
+// private
+bool kpMainWindow::open (const KURL &url, bool newDocSameNameIfNotExist)
+{
+    kpDocument *newDoc = openInternal (url,
+                                       defaultDocSize (),
+                                       newDocSameNameIfNotExist);
+    if (newDoc)
+    {
+        if (newDoc->isFromURL (false/*don't bother checking exists*/))
+            addRecentURL (url);
+        return true;
+    }
+    else
+    {
+        return false;
+    }
+}
+
+
 // private
 KURL::List kpMainWindow::askForOpenURLs (const QString &caption, const QString &startURL,
                                          bool allowMultipleURLs)
@@ -346,12 +385,179 @@ void kpMainWindow::slotOpenRecent (const KURL &url)
 {
 #if DEBUG_KP_MAIN_WINDOW
     kdDebug () << "kpMainWindow::slotOpenRecent(" << url << ")" << endl;
+    kdDebug () << "\titems=" << m_actionOpenRecent->items () << endl;
 #endif
 
     if (toolHasBegunShape ())
         tool ()->endShapeInternal ();
 
     open (url);
+
+    // If the open is successful, addRecentURL() would have bubbled up the
+    // URL in the File / Open Recent action.  As a side effect, the URL is
+    // deselected.
+    //
+    // If the open fails, we should deselect the URL:
+    //
+    // 1. for consistency
+    //
+    // 2. because it has not been opened.
+    //
+    m_actionOpenRecent->setCurrentItem (-1);
+}
+
+
+// private slot
+void kpMainWindow::slotScan ()
+{
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpMainWindow::slotScan() scanDialog=" << m_scanDialog << endl;
+#endif
+
+    if (toolHasBegunShape ())
+        tool ()->endShapeInternal ();
+
+
+    if (!m_scanDialog)
+    {
+        // Create scan dialog by looking for plugin.
+        // [takes about 500ms on 350Mhz]
+        m_scanDialog = KScanDialog::getScanDialog (this, "scandialog", true/*modal*/);
+
+        // No scanning support (kdegraphics/libkscan) installed?
+        // [Remove $KDEDIR/share/servicetypes/kscan.desktop and
+        //         $KDEDIR/share/services/scanservice.desktop to simulate this]
+        if (!m_scanDialog)
+        {
+        #if DEBUG_KP_MAIN_WINDOW
+            kdDebug () << "\tcould not create scan dialog" << endl;
+        #endif
+            // STRING: We really need a dialog here.  We can't due to string freeze.
+            //         In order to signal to the user -- without a dialog -- that
+            //         scanning support is not installed, we disable the action
+            //         in every window.
+            //
+            //         Or we could try to create the scan dialog in the ctor
+            //         and just disable the action in the first place.  But
+            //         this increases startup time and is also too risky on
+            //         the stable branch (e.g. if the scan support hangs,
+            //         KolourPaint would not be able to be started at all).
+
+            // TODO: PROPAGATE: interprocess
+            if (KMainWindow::memberList)
+            {
+                for (QPtrList <KMainWindow>::const_iterator it = KMainWindow::memberList->begin ();
+                     it != KMainWindow::memberList->end ();
+                     it++)
+                {
+                    kpMainWindow *mw = dynamic_cast <kpMainWindow *> (*it);
+                    Q_ASSERT (mw);
+
+                #if DEBUG_KP_MAIN_WINDOW
+                    kdDebug () << "\t\tmw=" << mw << endl;
+                #endif
+
+                    mw->m_actionScan->setEnabled (false);
+                }
+            }
+
+            return;
+        }
+
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tcreated scanDialog=" << m_scanDialog << endl;
+    #endif
+        connect (m_scanDialog, SIGNAL (finalImage (const QImage &, int)),
+                 SLOT (slotScanned (const QImage &, int)));
+    }
+
+
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\tcalling setup" << endl;
+#endif
+    // Bring up dialog to select scan device.
+    if (m_scanDialog->setup ())
+    {
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\t\tOK - showing dialog" << endl;
+    #endif
+        // Called only if scanner configured/available.
+        //
+        // In reality, this seems to be called even if you press "Cancel" in
+        // the KScanDialog::setup() dialog!
+        m_scanDialog->show ();
+    }
+    else
+    {
+        // Have never seen this code path execute even if "Cancel" is pressed.
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\t\tFAIL" << endl;
+    #endif
+    }
+}
+
+// private slot
+void kpMainWindow::slotScanned (const QImage &image, int)
+{
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpMainWindow::slotScanned() image.rect=" << image.rect () << endl;
+#endif
+
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\thiding dialog" << endl;
+#endif
+    // (KScanDialog does not close itself after a scan is made)
+    //
+    // Close the dialog, first thing:
+    //
+    // 1. This means that any dialogs we bring up won't be nested on top.
+    //
+    // 2. We don't want to return from this method but forget to close
+    //    the dialog.  So do it before anything else.
+    m_scanDialog->hide ();
+
+    // (just in case there's some drawing between slotScan() exiting and
+    //  us being called)
+    if (toolHasBegunShape ())
+        tool ()->endShapeInternal ();
+
+
+    // TODO: Maybe this code should be moved into kpdocument.cpp -
+    //       since it resembles the responsibilities of kpDocument::open().
+
+    // Convert QImage to kpDocument's image format, gathering meta info
+    // from QImage.
+    kpDocumentSaveOptions saveOptions;
+    kpDocumentMetaInfo metaInfo;
+    const QPixmap pixmap = kpDocument::convertToPixmapAsLosslessAsPossible (
+        image,
+        // STRING: After string freeze, have a custom message about the
+        //         "scanned image".
+        kpMainWindow::pasteWarnAboutLossInfo (),
+        &saveOptions,
+        &metaInfo);
+
+    if (pixmap.isNull ())
+    {
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tcould not convert to pixmap" << endl;
+    #endif
+        // STRING: After string freeze, we need a message like
+        //         "out of graphics memory" (see
+        //         kpDocument::getPixmapFromFile()).
+        return;
+    }
+
+
+    // Create document from image and meta info.
+    kpDocument *doc = new kpDocument (pixmap.width (), pixmap.height (), this);
+    doc->setPixmap (pixmap);
+    doc->setSaveOptions (saveOptions);
+    doc->setMetaInfo (metaInfo);
+
+
+    // Send document to current or new window.
+    setDocumentChoosingWindow (doc);
 }
 
 
@@ -362,7 +568,7 @@ bool kpMainWindow::save (bool localOnly)
         KImageIO::mimeTypes (KImageIO::Writing)
             .findIndex (m_document->saveOptions ()->mimeType ()) < 0 ||
         // SYNC: kpDocument::getPixmapFromFile() can't determine quality
-        //       from file
+        //       from file so it has been set initially to an invalid value.
         (m_document->saveOptions ()->mimeTypeHasConfigurableQuality () &&
             m_document->saveOptions ()->qualityIsInvalid ()) ||
         (localOnly && !m_document->url ().isLocalFile ()))
@@ -795,12 +1001,223 @@ void kpMainWindow::sendFilenameToPrinter (KPrinter *printer)
     }
 }
 
-// private
-void kpMainWindow::sendPixmapToPrinter (KPrinter *printer)
+
+static const double InchesPerMeter = 100 / 2.54;
+
+
+// TODO: GUI should allow viewing & changing of DPI.
+
+
+static bool shouldPrintImageCenteredOnPage ()
 {
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpmainwindow_file.cpp:shouldPrintImageCenteredOnPage()" << endl;
+#endif
+    bool ret;
+
+    KConfigGroupSaver cfgGroupSaver (KGlobal::config (),
+                                     kpSettingsGroupGeneral);
+    KConfigBase *cfg = cfgGroupSaver.config ();
+
+    if (cfg->hasKey (kpSettingPrintImageCenteredOnPage))
+    {
+        ret = cfg->readBoolEntry (kpSettingPrintImageCenteredOnPage);
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tread: " << ret << endl;
+    #endif
+    }
+    else
+    {
+        ret = true;
+#if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tfirst time - writing default: " << ret << endl;
+#endif
+        cfg->writeEntry (kpSettingPrintImageCenteredOnPage, ret);
+        cfg->sync ();
+    }
+
+    return ret;
+}
+
+
+// private
+void kpMainWindow::sendPixmapToPrinter (KPrinter *printer,
+        bool showPrinterSetupDialog)
+{
+    // Get image to be printed.
+    QPixmap pixmap = m_document->pixmapWithSelection ();
+
+
+    // Get image DPI.
+    double pixmapDotsPerMeterX =
+        double (m_document->metaInfo ()->dotsPerMeterX ());
+    double pixmapDotsPerMeterY =
+        double (m_document->metaInfo ()->dotsPerMeterY ());
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "kpMainWindow::sendPixmapToPrinter() pixmap:"
+               << " width=" << pixmap.width ()
+               << " height=" << pixmap.height ()
+               << " dotsPerMeterX=" << pixmapDotsPerMeterX
+               << " dotsPerMeterY=" << pixmapDotsPerMeterY
+               << endl;
+#endif
+
+    // Image DPI invalid (e.g. new image, could not read from file
+    // or Qt3 doesn't implement DPI for JPEG)?
+    if (pixmapDotsPerMeterX < 1 || pixmapDotsPerMeterY < 1)
+    {
+        // Even if just one DPI dimension is invalid, mutate both DPI
+        // dimensions as we have no information about the intended
+        // aspect ratio anyway (and other dimension likely to be invalid).
+
+        // When rendering text onto a document, the fonts are rasterised
+        // according to the screen's DPI.
+        // TODO: I think we should use the image's DPI.  Technically
+        //       possible?
+        //
+        //       So no matter what computer you draw text on, you get
+        //       the same pixels.
+        //
+        // So we must print at the screen's DPI to get the right text size.
+        //
+        // Unfortunately, this means that moving to a different screen DPI
+        // affects printing.  If you edited the image at a different screen
+        // DPI than when you print, you get incorrect results.  Furthermore,
+        // this is bogus if you don't have text in your image.  Worse still,
+        // what if you have multiple screens connected to the same computer
+        // with different DPIs?
+        // TODO: mysteriously, someone else is setting this to 96dpi always.
+        QPaintDeviceMetrics screenMetrics (&pixmap/*screen element*/);
+        const int dpiX = screenMetrics.logicalDpiX (),
+            dpiY = screenMetrics.logicalDpiY ();
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tusing screen dpi: x=" << dpiX << " y=" << dpiY << endl;
+    #endif
+
+        pixmapDotsPerMeterX = dpiX * InchesPerMeter;
+        pixmapDotsPerMeterY = dpiY * InchesPerMeter;
+    }
+
+
+    // Get page size (excluding margins).
+    // Coordinate (0,0) is the X here:
+    //     mmmmm
+    //     mX  m
+    //     m   m       m = margin
+    //     m   m
+    //     mmmmm
+    QPaintDeviceMetrics printerMetrics (printer);
+    const int printerWidthMM = printerMetrics.widthMM ();
+    const int printerHeightMM = printerMetrics.heightMM ();
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\tprinter: widthMM=" << printerWidthMM
+               << " heightMM=" << printerHeightMM
+               << endl;
+#endif
+
+
+    double dpiX = pixmapDotsPerMeterX / InchesPerMeter;
+    double dpiY = pixmapDotsPerMeterY / InchesPerMeter;
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\tpixmap: dpiX=" << dpiX << " dpiY=" << dpiY << endl;
+#endif
+
+
+    //
+    // If image doesn't fit on page at intended DPI, change the DPI.
+    //
+
+    const double scaleDpiX = (pixmap.width () / (printerWidthMM / 25.4))
+        / dpiX;
+    const double scaleDpiY = (pixmap.height () / (printerHeightMM / 25.4))
+        / dpiY;
+    const double scaleDpi = QMAX (scaleDpiX, scaleDpiY);
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\t\tscaleDpi: x=" << scaleDpiX << " y=" << scaleDpiY
+               << " --> scale at " << scaleDpi << " to fit?"
+               << endl;
+#endif
+
+    // Need to increase resolution to fit page?
+    if (scaleDpi > 1.0)
+    {
+        dpiX *= scaleDpi;
+        dpiY *= scaleDpi;
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\t\t\tto fit page, scaled to:"
+                   << " dpiX=" << dpiX << " dpiY=" << dpiY << endl;
+    #endif
+    }
+
+
+    // Make sure DPIs are equal as that's all QPrinter::setResolution()
+    // supports.  We do this in such a way that we only ever stretch an
+    // image, to avoid losing information.  Don't antialias as the printer
+    // will do that to translate our DPI to its physical resolution and
+    // double-antialiasing looks bad.
+    if (dpiX > dpiY)
+    {
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tdpiX > dpiY; stretching pixmap height to equalise DPIs to dpiX="
+                   << dpiX << endl;
+    #endif
+        kpPixmapFX::scale (&pixmap,
+             pixmap.width (),
+             QMAX (1, qRound (pixmap.height () * dpiX / dpiY)),
+             false/*don't antialias*/);
+
+        dpiY = dpiX;
+    }
+    else if (dpiY > dpiX)
+    {
+    #if DEBUG_KP_MAIN_WINDOW
+        kdDebug () << "\tdpiY > dpiX; stretching pixmap width to equalise DPIs to dpiY="
+                   << dpiY << endl;
+    #endif
+        kpPixmapFX::scale (&pixmap,
+             QMAX (1, qRound (pixmap.width () * dpiY / dpiX)),
+             pixmap.height (),
+             false/*don't antialias*/);
+
+        dpiX = dpiY;
+    }
+
+
+    // ASSERT: dpiX == dpiY
+    // QPrinter::setResolution() has to be called before QPrinter::setup().
+    printer->setResolution (QMAX (1, qRound (dpiX)));
+
+
+    double originX = 0, originY = 0;
+
+    // Centre image on page?
+    if (shouldPrintImageCenteredOnPage ())
+    {
+        originX = (printerWidthMM * dpiX / 25.4 - pixmap.width ()) / 2;
+        originY = (printerHeightMM * dpiY / 25.4 - pixmap.height ()) / 2;
+    }
+
+#if DEBUG_KP_MAIN_WINDOW
+    kdDebug () << "\torigin: x=" << originX << " y=" << originY << endl;
+#endif
+
+
+    sendFilenameToPrinter (printer);
+
+    if (showPrinterSetupDialog)
+    {
+        // The user can mutate margins at their own risk in this dialog.
+        // It doesn't seem to affect the size of the page as reported
+        // by QPaintDeviceMetrics::{width,height}MM().
+        if (!printer->setup (this))
+            return;
+    }
+
+
+    // Send pixmap to printer.
     QPainter painter;
     painter.begin (printer);
-    painter.drawPixmap (0, 0, m_document->pixmapWithSelection ());
+    painter.drawPixmap (qRound (originX), qRound (originY), pixmap);
     painter.end ();
 }
 
@@ -813,11 +1230,7 @@ void kpMainWindow::slotPrint ()
 
     KPrinter printer;
 
-    sendFilenameToPrinter (&printer);
-    if (!printer.setup (this))
-        return;
-
-    sendPixmapToPrinter (&printer);
+    sendPixmapToPrinter (&printer, true/*showPrinterSetupDialog*/);
 }
 
 // private slot
@@ -831,9 +1244,8 @@ void kpMainWindow::slotPrintPreview ()
 
     // TODO: pass "this" as parent
     printer.setPreviewOnly (true);
-    sendFilenameToPrinter (&printer);
 
-    sendPixmapToPrinter (&printer);
+    sendPixmapToPrinter (&printer, false/*don't showPrinterSetupDialog*/);
 }
 
 
@@ -1009,3 +1421,4 @@ void kpMainWindow::slotQuit ()
 
     close ();  // will call queryClose()
 }
+
