@@ -5,23 +5,32 @@
     copyright            : (C) 1999 by Klaas Freitag
     email                : freitag@suse.de
 
-    $Id: scanpackager.cpp,v 1.68.2.1 2003/03/22 19:51:53 faure Exp $
+    $Id: scanpackager.cpp 532728 2006-04-22 16:32:24Z pfeiffer $
  ***************************************************************************/
 
 /***************************************************************************
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *  This file may be distributed and/or modified under the terms of the    *
+ *  GNU General Public License version 2 as published by the Free Software *
+ *  Foundation and appearing in the file COPYING included in the           *
+ *  packaging of this file.                                                *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any version of the KADMOS ocr/icr engine of reRecognition GmbH,   *
+ *  Kreuzlingen and distribute the resulting executable without            *
+ *  including the source code for KADMOS in the source distribution.       *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any edition of Qt, and distribute the resulting executable,       *
+ *  without including the source code for Qt in the source distribution.   *
  *                                                                         *
  ***************************************************************************/
 
 #include "scanpackager.h"
-#include "miscwidgets.h"
 #include "resource.h"
 #include "img_saver.h"
 #include "kookaimage.h"
+#include "kookaimagemeta.h"
 #include "previewer.h"
 #include "devselector.h"
 
@@ -43,13 +52,14 @@
 #include <kurldrag.h>
 #include <kpopupmenu.h>
 #include <kaction.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 #include <kiconloader.h>
 #include <kfiledialog.h>
 #include <kurl.h>
 #include <kdebug.h>
 #include <klocale.h>
 #include <kglobal.h>
+#include <kio/global.h>
 #include <kio/progressbase.h>
 #include <kio/netaccess.h>
 #include <kio/jobclasses.h>
@@ -80,8 +90,8 @@ ScanPackager::ScanPackager( QWidget *parent ) : KFileTreeView( parent )
    setDropVisualizer(true);
    setAcceptDrops(true);
 
-   connect( this, SIGNAL(dropped( KURL::List&, KURL& )),
-	    this, SLOT( slotUrlsDropped( KURL::List&, KURL& )));
+   connect( this, SIGNAL(dropped( QWidget*, QDropEvent*, KURL::List&, KURL& )),
+	    this, SLOT( slotUrlsDropped( QWidget*, QDropEvent*, KURL::List&, KURL& )));
 
    kdDebug(28000) << "connected Drop-Signal" << endl;
    setRenameable ( 0, true );
@@ -90,20 +100,21 @@ ScanPackager::ScanPackager( QWidget *parent ) : KFileTreeView( parent )
    setRenameable ( 3, false );
 
    setRootIsDecorated( false );
-	
-   connect( this, SIGNAL( selectionChanged( QListViewItem*)),
-	    SLOT( slSelectionChanged(QListViewItem*)));
+
+   connect( this, SIGNAL( clicked( QListViewItem*)),
+	    SLOT( slClicked(QListViewItem*)));
 
    connect( this, SIGNAL( rightButtonPressed( QListViewItem *, const QPoint &, int )),
 	    SLOT( slShowContextMenue(QListViewItem *, const QPoint &, int )));
-	
+
    connect( this, SIGNAL(itemRenamed (QListViewItem*, const QString &, int ) ), this,
 	    SLOT(slFileRename( QListViewItem*, const QString&, int)));
 
-	
+
    img_counter = 1;
    /* Set the current export dir to home */
    m_currCopyDir = QDir::home().absPath();
+   m_currImportDir = m_currCopyDir;
 
    /* Preload frequently used icons */
    KIconLoader *loader = KGlobal::iconLoader();
@@ -113,7 +124,7 @@ ScanPackager::ScanPackager( QWidget *parent ) : KFileTreeView( parent )
    m_colorPixmap  = loader->loadIcon( "palette_color", KIcon::Small );
 
    m_startup = true;
-   
+
    /* create a context menu and set the title */
    m_contextMenu = new KPopupMenu();
    static_cast<KPopupMenu*>(m_contextMenu)->insertTitle( i18n( "Gallery" ));
@@ -140,19 +151,19 @@ KFileTreeBranch* ScanPackager::openRoot( const KURL& root, bool  )
    m_defaultBranch = addBranch( root, i18n("Kooka Gallery"),
 				loader->loadIcon( "folder_image", KIcon::Small ),
 				false /* do not showHidden */ );
-   
+
    // Q_CHECK_PTR( m_defaultBranch );
    m_defaultBranch->setOpenPixmap( loader->loadIcon( "folder_blue_open", KIcon::Small ));
-   
+
    setDirOnlyMode( m_defaultBranch, false );
    m_defaultBranch->setShowExtensions( true ); // false );
-   
+
    connect( m_defaultBranch, SIGNAL( newTreeViewItems( KFileTreeBranch*, const KFileTreeViewItemList& )),
 	    this, SLOT( slotDecorate(KFileTreeBranch*, const KFileTreeViewItemList& )));
 
    connect( m_defaultBranch, SIGNAL( directoryChildCount( KFileTreeViewItem* , int )),
 	    this, SLOT( slotDirCount( KFileTreeViewItem *, int )));
-   
+
    connect( m_defaultBranch, SIGNAL( deleteItem( KFileItem* )),
 	    this, SLOT( slotDeleteFromBranch(KFileItem*)));
 
@@ -183,7 +194,7 @@ void ScanPackager::slotDirCount( KFileTreeViewItem* item, int cnt )
 {
    if( item && item->isDir() )
    {
-      QString cc = i18n( "%1 items").arg( cnt );
+      QString cc = i18n( "one item", "%n items", cnt);
       item->setText( 1, cc );
    }
    else
@@ -206,12 +217,12 @@ void ScanPackager::slotDecorate( KFileTreeViewItem* item )
       KFileItem *kfi = item->fileItem();
 
       KookaImage *img = 0L;
-      
+
       if( kfi )
       {
 	 img = static_cast<KookaImage*>(kfi->extraData( this ));
       }
-      
+
       if( img )
       {
 
@@ -232,7 +243,7 @@ void ScanPackager::slotDecorate( KFileTreeViewItem* item )
 	       item->setPixmap( 0, m_colorPixmap );
 	    }
 	 }
-	 
+
 	 /* set image size in pixels */
 	 QString t = i18n( "%1 x %2" ).arg( img->width()).arg(img->height());
 	 item->setText( 1, t );
@@ -242,26 +253,29 @@ void ScanPackager::slotDecorate( KFileTreeViewItem* item )
       {
 	 /* Item is not yet loaded. Display file information */
 	 item->setPixmap( 0, m_floppyPixmap );
-	 KIO::filesize_t s = kfi->size();
-	 double dsize = s / 1024.0;  /* calc kilobytes */
-
-	 QString sizeStr;
-	 if( dsize > 999.99 )
+	 if ( kfi )
 	 {
-	    dsize = dsize / 1024.0;
-	    sizeStr = i18n( "%1 MB").arg( dsize, 0, 'f', 2 );
+	    item->setText(1, KIO::convertSize( kfi->size() ));
 	 }
-	 else
-	 {
-	    sizeStr = i18n( "%1 kB").arg( dsize, 0,'f', 1 );
-	 }
-	 item->setText(1, sizeStr );
       }
 
       /* Image format */
       QString format = getImgFormat( item );
       item->setText( 2, format );
    }
+
+   // This code is quite similar to m_nextUrlToSelect in KFileTreeView::slotNewTreeViewItems
+   // When scanning a new image, we wait for the KDirLister to notice the new file,
+   // and then we have the KFileTreeViewItem that we need to display the image.
+   if ( ! m_nextUrlToShow.isEmpty() )
+   {
+       if( m_nextUrlToShow.equals(item->url(), true ))
+       {
+           m_nextUrlToShow = KURL(); // do this first to prevent recursion
+           slClicked( item );
+           setCurrentItem(item);     // neccessary in case of new file from D&D
+       }
+   }
 }
 
 
@@ -293,7 +307,7 @@ void ScanPackager::slFileRename( QListViewItem* it, const QString& newStr, int )
 
    if( newStr.isEmpty() )
       success = false;
-   
+
    KFileTreeViewItem *item = static_cast<KFileTreeViewItem*>(it);
 
    /* Free memory and imform everybody who is interested. */
@@ -320,12 +334,14 @@ void ScanPackager::slFileRename( QListViewItem* it, const QString& newStr, int )
 
 	 kdDebug(28000) << "Renaming to " << urlTo.prettyURL() <<
 	    " from " << urlFrom.prettyURL() << endl;
-   
+
 	 /* to urlTo the really used filename is written */
 	 setSelected( item, false );
+
 	 if( ImgSaver::renameImage( urlFrom, urlTo, false, this ) )
-	 {      
+	 {
 	    kdDebug(28000) << "renaming OK" << endl;
+            emit fileRenamed( item->fileItem(), urlTo );
 	    success=true;
 	 }
 	 else
@@ -335,15 +351,15 @@ void ScanPackager::slFileRename( QListViewItem* it, const QString& newStr, int )
       }
    }
 
-   if( ! success )
-   {      
+   if( !success )
+   {
       kdDebug(28000) << "renaming failed" << endl;
       /* restore the name */
       item->setText(0, urlFrom.fileName() );
       setSelected( item, true );
 
    }
-      
+
 }
 
 
@@ -366,8 +382,8 @@ QString ScanPackager::buildNewFilename( QString cmplFilename, QString currFormat
    QString ext = "";
 
    kdDebug(28000) << "Filename wanted: "<< cmplFilename << " <"<<newExt<<"> <" << nowExt<<">" <<endl;
-   
-   if( newExt == "" )
+
+   if( newExt.isEmpty() )
    {
       /* ok, fine -> return the currFormat-Extension */
       ext = base + "." + currFormat;
@@ -396,11 +412,11 @@ QString ScanPackager::itemDirectory( const KFileTreeViewItem* item, bool relativ
    if( ! item )
    {
       kdDebug(28000) << "ERR: itemDirectory without item" << endl;
-      
+      return QString::null;
    }
 
    QString relativUrl= (item->url()).prettyURL();
-   
+
    if( ! item->isDir() )
    {
       // Cut off the filename in case it is not a dir
@@ -412,7 +428,7 @@ QString ScanPackager::itemDirectory( const KFileTreeViewItem* item, bool relativ
       if( ! relativUrl.endsWith( "/" ) )
 	 relativUrl.append( "/" );
    }
-      
+
    if( relativ )
    {
       KFileTreeBranch *branch = item->branch();
@@ -420,12 +436,12 @@ QString ScanPackager::itemDirectory( const KFileTreeViewItem* item, bool relativ
       {
 	 kdDebug(28000) << "Relativ URL of the file " << relativUrl << endl;
 	 QString rootUrl = (branch->rootUrl()).prettyURL();  // directory of branch root
-      
+
 	 if( relativUrl.startsWith( rootUrl ))
 	 {
 	    relativUrl.remove( 0, rootUrl.length() );
 
-	    if( relativUrl == "" ) relativUrl = "/"; // The root
+	    if( relativUrl.isEmpty() ) relativUrl = "/"; // The root
 	 }
 	 else
 	 {
@@ -439,10 +455,10 @@ QString ScanPackager::itemDirectory( const KFileTreeViewItem* item, bool relativ
 /* This slot receives a string from the gallery-path combobox shown under the
  * image gallery. The form of the string coming in here is <branch-name> - <
  * relativ directory under the branch. Now it is to assemble a complete path
- * from the data, find out which KFileTreeViewITem is associated with it and
- * call slSelectionChanged with it.
+ * from the data, find out which KFileTreeViewItem is associated with it and
+ * call slClicked with it.
  */
- 
+
 void ScanPackager::slotSelectDirectory( const QString & dirString )
 {
    kdDebug(28000) << "Trying to decode directory string " << dirString << endl;
@@ -455,41 +471,42 @@ void ScanPackager::slotSelectDirectory( const QString & dirString )
       /* Splitting up the string coming in */
       QString branchName = dirString.left( pos );
       QString relPath( dirString );
-      
+
       relPath = relPath.remove( 0, pos + searchFor.length());
 
       kdDebug(28000) << "Splitted up to branch <" << branchName << "> and <" << relPath << endl;
-      
+
       KFileTreeViewItem *kfi = findItem( branchName, relPath );
-      
+
       if( kfi )
       {
 	 kdDebug(28000) << "got a new item to select !" << endl;
 	 ensureItemVisible(kfi);
 	 setCurrentItem(kfi);
+         slClicked(kfi); // load thumbnails for this dir etc.
       }
    }
 }
 
 /* ----------------------------------------------------------------------- */
-/* This slot is called if the selection changes.                           */
-void ScanPackager::slSelectionChanged( QListViewItem *newItem )
+/* This slot is called when clicking on an item.                           */
+void ScanPackager::slClicked( QListViewItem *newItem )
 {
    KFileTreeViewItem *item = static_cast<KFileTreeViewItem*>(newItem);
 
-   if( item )
+   if( item ) // can be 0, when clicking where no item is present
    {
-      kdDebug(28000) << "SelectionChanged - newItem !" << endl;
+      kdDebug(28000) << "Clicked - newItem !" << endl;
       /* Check if directory, hide image for now, later show a thumb view */
       if( item->isDir())
       {
-	 kdDebug(28000) << "selectionChanged: Is a directory !" << endl;
+	 kdDebug(28000) << "clicked: Is a directory !" << endl;
 	 emit( showImage( 0L ));
 	 kdDebug(28000) << "emitting showThumbnails" << endl;
       }
       else
       {
-	 /* if not a dir, load the image if neccessary. This is done by loadImageForItem,
+	 /* if not a dir, load the image if necessary. This is done by loadImageForItem,
 	  * which is async( TODO ). The image finally arrives in slotImageArrived */
 	 QApplication::setOverrideCursor(waitCursor);
 	 emit( aboutToShowImage( item->url()));
@@ -499,7 +516,7 @@ void ScanPackager::slSelectionChanged( QListViewItem *newItem )
 
       /* emit a signal indicating the new directory if there is a new one */
       QString wholeDir = itemDirectory( item, false ); /* not relativ to root */
-      
+
       if( currSelectedDir != wholeDir )
       {
 	 currSelectedDir = wholeDir;
@@ -519,24 +536,20 @@ void ScanPackager::slSelectionChanged( QListViewItem *newItem )
       }
       else
       {
-	 // kdDebug(28000) << "directroy is not new: " << currSelectedDir << endl;
+	 // kdDebug(28000) << "directory is not new: " << currSelectedDir << endl;
       }
    }
-   else
-   {
-      kdDebug(28000) << "ERR: Failed to get a valid treeviewitem" << endl;
-   }
 }
 
 void ScanPackager::loadImageForItem( KFileTreeViewItem *item )
 {
-   
+
    if( ! item ) return;
    bool result = true;
-   
+
    KFileItem *kfi = item->fileItem();
    if( ! kfi ) return;
-   
+
    KookaImage *img = static_cast<KookaImage*>( kfi->extraData(this));
 
    if( img )
@@ -552,8 +565,8 @@ void ScanPackager::loadImageForItem( KFileTreeViewItem *item )
        * them.
        */
       KURL url = item->url();
-   
-      img = new KookaImage( ); 
+
+      img = new KookaImage( );
       if( !img || !img->loadFromUrl( url ) )
       {
 	 kdDebug(28000) << "Loading KookaImage from File failed!" << endl;
@@ -561,39 +574,42 @@ void ScanPackager::loadImageForItem( KFileTreeViewItem *item )
       }
       else
       {
-	 /* care for subimages, create items for them */
-	 kdDebug(28000) << "subImage-count: " << img->subImagesCount() << endl;
-	 if( img->subImagesCount() > 1 )
-	 {
-	    KIconLoader *loader = KGlobal::iconLoader();
-	    kdDebug(28000) << "SubImages existing!" << endl;
+          /* store the fileitem */
+          img->setFileItem( kfi );
 
-	    /* Start at the image with index 1, that makes  one less than are actually in the
-	     * image. But image 0 was already created above. */
-	    KFileTreeViewItem *prevItem=0;
-	    for( int i = 1; i < img->subImagesCount(); i++ )
-	    {
-	       kdDebug(28000) << "Creating subimage no " << i << endl;
-	       KFileItem *newKfi = new KFileItem( *kfi );
-	       KFileTreeViewItem *subImgItem = new KFileTreeViewItem( item, newKfi, item->branch());
+          /* care for subimages, create items for them */
+          kdDebug(28000) << "subImage-count: " << img->subImagesCount() << endl;
+          if( img->subImagesCount() > 1 )
+          {
+              KIconLoader *loader = KGlobal::iconLoader();
+              kdDebug(28000) << "SubImages existing!" << endl;
 
-	       if( prevItem )
-	       {
-		  subImgItem->moveItem( prevItem );
-	       }
-	       prevItem = subImgItem;
+              /* Start at the image with index 1, that makes  one less than are actually in the
+               * image. But image 0 was already created above. */
+              KFileTreeViewItem *prevItem=0;
+              for( int i = 1; i < img->subImagesCount(); i++ )
+              {
+                  kdDebug(28000) << "Creating subimage no " << i << endl;
+                  KFileItem *newKfi = new KFileItem( *kfi );
+                  KFileTreeViewItem *subImgItem = new KFileTreeViewItem( item, newKfi, item->branch());
 
-	       subImgItem->setPixmap( 0, loader->loadIcon( "editcopy", KIcon::Small ));
-	       subImgItem->setText( 0, i18n("Sub-image %1").arg( i ) );
-	       KookaImage  *subImgImg = new KookaImage( i, img );
-	    
-	       newKfi->setExtraData( (void*) this, (void*) subImgImg );
-	    }
-	 }
+                  if( prevItem )
+                  {
+                      subImgItem->moveItem( prevItem );
+                  }
+                  prevItem = subImgItem;
+
+                  subImgItem->setPixmap( 0, loader->loadIcon( "editcopy", KIcon::Small ));
+                  subImgItem->setText( 0, i18n("Sub-image %1").arg( i ) );
+                  KookaImage  *subImgImg = new KookaImage( i, img );
+                  subImgImg->setFileItem( newKfi );
+                  newKfi->setExtraData( (void*) this, (void*) subImgImg );
+              }
+          }
       }
    }
 
-   
+
    if( result && img )
    {
       if( img->isSubImage() )
@@ -630,12 +646,27 @@ void ScanPackager::slImageArrived( KFileTreeViewItem *item, KookaImage* image )
    }
 }
 
+KookaImage* ScanPackager::getCurrImage() const
+{
+    KFileTreeViewItem *curr = currentKFileTreeViewItem();
+    KookaImage *img = 0L;
+
+    if( curr )
+    {
+        KFileItem *kfi = curr->fileItem();
+        if( kfi )
+        {
+          img = static_cast<KookaImage*>(kfi->extraData( this ));
+        }
+    }
+    return(img);
+}
 
 
 QString ScanPackager::getCurrImageFileName( bool withPath = true ) const
 {
    QString result = "";
-   
+
    KFileTreeViewItem *curr = currentKFileTreeViewItem();
    if( ! curr )
    {
@@ -658,24 +689,24 @@ QString ScanPackager::getCurrImageFileName( bool withPath = true ) const
 }
 
 /* ----------------------------------------------------------------------- */
-QCString ScanPackager::getImgFormat( KFileTreeViewItem* item ) const 
+QCString ScanPackager::getImgFormat( KFileTreeViewItem* item ) const
 {
 
    QCString cstr;
-   
+
    if( !item ) return( cstr );
 #if 0
    KFileItem *kfi = item->fileItem();
-   
+
    QString mime = kfi->mimetype();
 #endif
-   
+
    // TODO find the real extension for use with the filename !
    // temporarely:
    QString f = localFileName( item );
 
    return( QImage::imageFormat( f ));
-   
+
 }
 
 QString ScanPackager::localFileName( KFileTreeViewItem *it ) const
@@ -706,10 +737,10 @@ void ScanPackager::slotCurrentImageChanged( QImage *img )
 
    /* Do not save directories */
    if( curr->isDir() ) return;
-   
+
    /* unload image and free memory */
    slotUnloadItem( curr );
-   
+
    const QString filename = localFileName( curr );
    const QCString format = getImgFormat( curr );
    ImgSaver saver( this );
@@ -752,7 +783,7 @@ void ScanPackager::slotCurrentImageChanged( QImage *img )
 /* This slot takes a new scanned Picture and saves it.
  * It urgently needs to make a deep copy of the image !
  */
-void ScanPackager::slAddImage( QImage *img )
+void ScanPackager::slAddImage( QImage *img, KookaImageMeta* )
 {
    ImgSaveStat is_stat = ISS_OK;
    /* Save the image with the help of the ImgSaver */
@@ -765,7 +796,7 @@ void ScanPackager::slAddImage( QImage *img )
    if( ! curr )
    {
       KFileTreeBranch *b = branches().at(0); /* There should be at least one */
-      
+
       if( b )
       {
 	 curr = findItem( b, i18n( "Incoming/" ) );
@@ -779,7 +810,7 @@ void ScanPackager::slAddImage( QImage *img )
    }
 
    /* find the directory above the current one */
-   
+
    KURL dir(itemDirectory( curr ));
 
    /* Path of curr sel item */
@@ -809,23 +840,24 @@ void ScanPackager::slAddImage( QImage *img )
 
    /* Add the new image to the list of new images */
    KURL lurl = img_saver.lastFileUrl();
-   
+
    KFileTreeBranchList branchlist = branches();
    KFileTreeBranch *kookaBranch = branchlist.at(0);
 
    QString strdir = itemDirectory(curr);
    if(strdir.endsWith(QString("/"))) strdir.truncate( strdir.length() - 1 );
    kdDebug(28000) << "Updating directory with " << strdir << endl;
-   
+
    if( kookaBranch ) kookaBranch->updateDirectory( KURL(strdir) );
    slotSetNextUrlToSelect( lurl );
-   
+   m_nextUrlToShow = lurl;
+
    QString s;
    /* Count amount of children of the father */
    QListViewItem *paps = curr->parent();
    if( curr->isDir() ) /* take only father if the is no directory */
       paps = curr;
-   
+
    if( paps )
    {
       int childcount = paps->childCount();
@@ -850,7 +882,7 @@ void ScanPackager::slSelectImage( const KURL& name )
       kdDebug(28000) << "slSelectImage: Found an item !" << endl;
       ensureItemVisible( found );
       setCurrentItem( found );
-      slSelectionChanged( found );
+      slClicked( found );
    }
 
 }
@@ -872,8 +904,8 @@ KFileTreeViewItem *ScanPackager::spFindItem( SearchType type, const QString name
    {
       branchList = branches();
    }
- 
-   
+
+
    KFileTreeBranchIterator it( branchList );
    KFileItem *kfi = 0L;
    KFileTreeViewItem *foundItem = 0L;
@@ -896,12 +928,12 @@ KFileTreeViewItem *ScanPackager::spFindItem( SearchType type, const QString name
 	 case UrlSearch:
 	    kdDebug(28000) << "ScanPackager: URL search for " << name << endl;
 	    kfi = branchloop->find( url );
-	    break;	
+	    break;
       default:
 	 kdDebug(28000) << "Scanpackager: Wrong search type !" << endl;
 	 break;
       }
-	 
+
    }
    if( kfi )
    {
@@ -939,7 +971,7 @@ void ScanPackager::slotExportFile( )
 {
    KFileTreeViewItem *curr = currentKFileTreeViewItem();
    if( ! curr ) return;
-   
+
    if( curr->isDir() )
    {
       kdDebug(28000) << "Not yet implemented!" << endl;
@@ -949,7 +981,7 @@ void ScanPackager::slotExportFile( )
       KURL fromUrl( curr->url());
       QString filter = "*." + getImgFormat(curr).lower();
       filter += "\n*|" + i18n( "All Files" );
-      
+
       // initial += fromUrl.filename(false);
       QString initial = m_currCopyDir + "/";
       initial += fromUrl.filename(false);
@@ -961,8 +993,8 @@ void ScanPackager::slotExportFile( )
          if( fromUrl == fileName ) return;
 
 	 /* Since it is asynchron, we will never get if it succeeded. */
-	 ImgSaver::exportImage( fromUrl, fileName );
-	 
+	 ImgSaver::copyImage( fromUrl, fileName );
+
          /* remember the filename for the next export */
 	 fileName.setFileName( QString());
 	 m_currCopyDir = fileName.url( );
@@ -970,31 +1002,57 @@ void ScanPackager::slotExportFile( )
    }
 }
 
-void ScanPackager::storeJob( KIO::Job *job, KFileTreeViewItem *item, JobDescription::JobType jType )
-{
-   JobDescription newJob ( job, item, jType );
-   jobMap.insert( job, newJob );
-}
 
-void ScanPackager::slotExportFinished( KIO::Job *job )
+void ScanPackager::slotImportFile()
 {
-   // nothing yet to do.
-   kdDebug(28000) << "Export Finished" << endl;
-   jobMap.remove(job);
+    KFileTreeViewItem *curr = currentKFileTreeViewItem();
+    if( ! curr ) return;
+
+    KURL impTarget = curr->url();
+
+    if( ! curr->isDir() )
+    {
+        KFileTreeViewItem *pa = static_cast<KFileTreeViewItem*>(curr->parent());
+        impTarget = pa->url();
+    }
+    kdDebug(28000) << "Importing to " << impTarget.url() << endl;
+
+    KURL impUrl = KFileDialog::getImageOpenURL ( m_currImportDir, this, i18n("Import Image File to Gallery"));
+
+    if( ! impUrl.isEmpty() )
+    {
+        m_currImportDir = impUrl.url();
+        impTarget.addPath( impUrl.fileName());  // append the name of the sourcefile to the path
+        m_nextUrlToShow = impTarget;
+        ImgSaver::copyImage( impUrl, impTarget );
+    }
 }
 
 
-void ScanPackager::slotUrlsDropped( KURL::List& urls, KURL& copyTo )
-{
-   kdDebug(28000) << "Kooka drop event" << endl;
 
-   if( urls.count() > 0 )
+void ScanPackager::slotUrlsDropped( QWidget*, QDropEvent* ev, KURL::List& urls, KURL& copyTo )
+{
+   if( !urls.isEmpty() )
    {
-      copyjob = KIO::copy( urls, copyTo, true );
-      KFileTreeViewItem *curr = currentKFileTreeViewItem();
-      storeJob( copyjob, curr, JobDescription::ImportJob );
-      connect( copyjob, SIGNAL(result(KIO::Job*)),
-	       this, SLOT(slotExportFinished( KIO::Job* )));
+       kdDebug(28000) << "Kooka drop event!" << endl;
+       // kdDebug(28000) << "Kooka drop event. First src url=" << urls.first() << " copyTo=" << copyTo
+       //    << " move=" << ( ev->action() == QDropEvent::Move ) << endl;
+
+       /* first make the last url to copy to the one to select next */
+       if( ! urls.empty() )
+       {
+           KURL nextSel = copyTo;
+           nextSel.addPath( urls.back().fileName(false));
+
+           kdDebug(28000) << "Selecting next url: " << nextSel.url() << endl;
+           m_nextUrlToShow = nextSel;
+           // slotSetNextUrlToSelect( nextSel );
+       }
+
+      if ( ev->action() == QDropEvent::Move )
+        copyjob = KIO::move( urls, copyTo, true );
+      else
+        copyjob = KIO::copy( urls, copyTo, true );
    }
 }
 
@@ -1015,7 +1073,7 @@ void ScanPackager::slotUnloadItems( )
 void ScanPackager::slotUnloadItem( KFileTreeViewItem *curr )
 {
    if( ! curr ) return;
-   
+
    if( curr->isDir())
    {
       KFileTreeViewItem *child = static_cast<KFileTreeViewItem*>(curr->firstChild());
@@ -1037,7 +1095,7 @@ void ScanPackager::slotUnloadItem( KFileTreeViewItem *curr )
 	 if( image->subImagesCount() > 0 )
 	 {
 	    KFileTreeViewItem *child = static_cast<KFileTreeViewItem*>(curr->firstChild());
-	 
+
 	    while( child )
 	    {
 	       KFileTreeViewItem *nextChild = 0;
@@ -1048,7 +1106,7 @@ void ScanPackager::slotUnloadItem( KFileTreeViewItem *curr )
 	       child = nextChild;
 	    }
 	 }
-      
+
 	 emit( unloadImage( image ));
 	 delete image;
 	 kfi->removeExtraData( this );
@@ -1065,12 +1123,12 @@ void ScanPackager::slotDeleteItems( )
 
    KURL urlToDel = curr->url();
    QListViewItem *nextToSelect = curr->nextSibling();
-   
+
    kdDebug(28000) << "Deleting: " << urlToDel.prettyURL() << endl;
    bool ask = true; /* for later use */
 
    int result = KMessageBox::Yes;
-   
+
    KFileItem *item = curr->fileItem();
    if( ask )
    {
@@ -1080,15 +1138,14 @@ void ScanPackager::slotDeleteItems( )
       {
 	 s = i18n("Do you really want to delete the folder %1\nand all the images inside?").arg("");
       }
-      result = KMessageBox::questionYesNo(this, s, i18n( "Delete Collection Item"),
-					  KStdGuiItem::yes(), KStdGuiItem::no(),
-					  "AskForDeleteFiles" );
+      result = KMessageBox::warningContinueCancel(this, s, i18n( "Delete Collection Item"),
+					  KStdGuiItem::del(), "AskForDeleteFiles" );
    }
 
    /* Since we are currently talking about local files here, NetAccess is OK */
-   if( result == KMessageBox::Yes )
+   if( result == KMessageBox::Continue )
    {
-      if( KIO::NetAccess::del( urlToDel ))
+      if( KIO::NetAccess::del( urlToDel, 0 ))
       {
 	 if( nextToSelect )
 	    setSelected( nextToSelect, true );
@@ -1098,25 +1155,24 @@ void ScanPackager::slotDeleteItems( )
 	    /* The directory needs to be removed from the name combo */
 	    emit(directoryToRemove( curr->branch(), itemDirectory( curr, true ) ));
 	 }
-	 
+
       }
       else
 	 kdDebug(28000) << "Deleting files failed" << endl;
-	 
+
    }
 }
 
 /* ----------------------------------------------------------------------- */
 void ScanPackager::slotCreateFolder( )
 {
-   KLineEditDlg d(i18n("Please enter a name for the new folder:"), QString::null, this);
-   d.setCaption(i18n("New Folder"));
+   bool ok;
+   QString folder = KInputDialog::getText( i18n( "New Folder" ),
+         i18n( "Please enter a name for the new folder:" ), QString::null,
+         &ok, this );
 
-   if( d.exec() )
+   if( ok )
    {
-      QString folder = d.text();
-      if( folder.length() > 0 )
-      {
 	 /* KIO create folder goes here */
 
 	 KFileTreeViewItem *it = currentKFileTreeViewItem();
@@ -1137,18 +1193,17 @@ void ScanPackager::slotCreateFolder( )
 	     */
 	    slotSetNextUrlToSelect( url );
 
-	    if( ! KIO::NetAccess::mkdir( url ))
+	    if( ! KIO::NetAccess::mkdir( url, 0, -1 ))
 	    {
 	       kdDebug(28000) << "ERR: creation of " << url.prettyURL() << " failed !" << endl;
 	    }
 	    else
 	    {
 	       /* created successfully */
-	       /* open the branch if neccessary and select the new folder */
+	       /* open the branch if necessary and select the new folder */
 
 	    }
 	 }
-      }
    }
 }
 
@@ -1160,7 +1215,7 @@ QString ScanPackager::getImgName( QString name_on_disk )
    (void) name_on_disk;
 
    s = i18n("image %1").arg(img_counter++);
-   return( s );     	
+   return( s );
 }
 
 /* ----------------------------------------------------------------------- */
@@ -1177,24 +1232,29 @@ void ScanPackager::slotDeleteFromBranch( KFileItem* kfi )
 
 void ScanPackager::contentsDragMoveEvent( QDragMoveEvent *e )
 {
-   if( !e || ! acceptDrag( e ) )
+   if( ! acceptDrag( e ) )
    {
       e->ignore();
       return;
    }
 
-   QListViewItem *item = 0;
-   QListViewItem *par = 0;
+   QListViewItem *afterme = 0;
+   QListViewItem *parent = 0;
 
-   findDrop( e->pos(), par, item );
+   findDrop( e->pos(), parent, afterme );
+
+   // "afterme" is 0 when aiming at a directory itself
+   QListViewItem *item = afterme ? afterme : parent;
 
    if( item )
    {
       bool isDir = static_cast<KFileTreeViewItem*> (item)->isDir();
-      if( isDir )
-	 KFileTreeView::contentsDragMoveEvent( e );
+      if( isDir ) {
+         KFileTreeView::contentsDragMoveEvent( e ); // for the autoopen code
+         return;
+      }
    }
-
+   e->acceptAction();
 }
 
 
