@@ -15,6 +15,7 @@
 #include <qapplication.h>
 #include <qpaintdevicemetrics.h>
 #include <qregexp.h>
+#include <qvariant.h>
 #include <kapplication.h>
 #include <klistview.h>
 #include <klocale.h>
@@ -35,7 +36,6 @@
 #include "xpdf/ErrorCodes.h"
 #include "xpdf/UnicodeMap.h"
 #include "xpdf/Outline.h"
-#include "xpdf/UGString.h"
 #include "goo/GList.h"
 
 // local includes
@@ -46,6 +46,8 @@
 #include "core/pagetransition.h"
 #include "conf/settings.h"
 
+#include <stdlib.h>
+
 #include <config.h>
 
 // id for DATA_READY PDFPixmapGeneratorThread Event
@@ -315,7 +317,8 @@ void PDFGenerator::generatePixmap( PixmapRequest * request )
     // note: thread safety is set on 'false' for the GUI (this) thread
     kpdfOutputDev->setParams( request->width, request->height, genObjectRects, genObjectRects, false );
     pdfdoc->displayPage( kpdfOutputDev, page->number() + 1, fakeDpiX, fakeDpiY, 0, false, true, genObjectRects );
-    delete pdfdoc->takeLinks(); // we have already created the KPDFLinks so free the xpdf memory
+    if ( genObjectRects )
+        pdfdoc->processLinks( kpdfOutputDev, page->number() + 1 );
 
     // 2. Take data from outputdev and attach it to the Page
     page->setPixmap( request->id, kpdfOutputDev->takePixmap() );
@@ -408,6 +411,7 @@ void PDFGenerator::putFontInfo(KListView *list)
 
 bool PDFGenerator::print( KPrinter& printer )
 {
+    // PageSize is a CUPS artificially created setting
     QString ps = printer.option("PageSize");
     int paperWidth, paperHeight;
     int marginTop, marginLeft, marginRight, marginBottom;
@@ -415,10 +419,11 @@ bool PDFGenerator::print( KPrinter& printer )
     marginLeft = (int)printer.option("kde-margin-left").toDouble();
     marginRight = (int)printer.option("kde-margin-right").toDouble();
     marginBottom = (int)printer.option("kde-margin-bottom").toDouble();
+    bool forceRasterize = printer.option("kde-kpdf-forceRaster").toInt();
 
     if (ps.find(QRegExp("w\\d+h\\d+")) == 0)
     {
-        // size not supported by Qt, KPrinter gives us the size as wWIDTHhHEIGHT
+        // size not supported by Qt, CUPS gives us the size as wWIDTHhHEIGHT, at least on the printers i tester
         // remove the w
         ps = ps.mid(1);
         int hPos = ps.find("h");
@@ -427,10 +432,15 @@ bool PDFGenerator::print( KPrinter& printer )
     }
     else
     {
-        // size is supported by Qt, we get either the pageSize name or nothing because the default pageSize is used
+        // size is supported by Qt, we get either the pageSize name or nothing because the CUPS driver
+        // does not do any translation, then use KPrinter::pageSize to get the page size
+        KPrinter::PageSize qtPageSize;
+        if (!ps.isEmpty()) qtPageSize = pageNameToPageSize(ps);
+        else qtPageSize = printer.pageSize();
+
         QPrinter dummy(QPrinter::PrinterResolution);
         dummy.setFullPage(true);
-        dummy.setPageSize((QPrinter::PageSize)(ps.isEmpty() ? KGlobal::locale()->pageSize() : pageNameToPageSize(ps)));
+        dummy.setPageSize((QPrinter::PageSize)qtPageSize);
 
         QPaintDeviceMetrics metrics(&dummy);
         paperWidth = metrics.width();
@@ -445,18 +455,26 @@ bool PDFGenerator::print( KPrinter& printer )
     {
         pstitle = m_document->currentDocument().fileName( false );
     }
-    const char* pstitlechar = !pstitle.isEmpty() ? pstitle.local8Bit() : 0;
-    PSOutputDev *psOut = new PSOutputDev(tf.name().latin1(), pstitlechar, pdfdoc->getXRef(), pdfdoc->getCatalog(), 1, pdfdoc->getNumPages(), psModePS, marginRight, marginBottom, paperWidth - marginLeft, paperHeight - marginTop);
+    QCString pstitle8Bit = pstitle.local8Bit();
+    const char* pstitlechar;
+    if (!pstitle.isEmpty())
+    {
+      pstitlechar = pstitle8Bit.data();
+      printer.setDocName(pstitle);
+    }
+    else
+    {
+      pstitlechar = 0;
+    }
+    PSOutputDev *psOut = new PSOutputDev(const_cast<char*>(tf.name().latin1()), const_cast<char*>(pstitlechar), pdfdoc->getXRef(), pdfdoc->getCatalog(), 1, pdfdoc->getNumPages(), psModePS, marginRight, marginBottom, paperWidth - marginLeft, paperHeight - marginTop, forceRasterize);
 
     if (psOut->isOk())
     {
-        std::list<int> pages;
-
         double xScale = ((double)paperWidth - (double)marginLeft - (double)marginRight) / (double)paperWidth;
         double yScale = ((double)paperHeight - (double)marginBottom - (double)marginTop) / (double)paperHeight;
 
         if ( abs((int)(xScale * 100) - (int)(yScale * 100)) > 5 ) {
-            int result = KMessageBox::questionYesNo(0,
+            int result = KMessageBox::questionYesNo(m_document->widget(),
                                        i18n("The margins you specified change the page aspect ratio. Do you want to print with the aspect ratio changed or do you want the margins to be adapted so that the aspect ratio is preserved?"),
                                        i18n("Aspect ratio change"),
                                        i18n("Print with specified margins"),
@@ -465,20 +483,23 @@ bool PDFGenerator::print( KPrinter& printer )
             if (result == KMessageBox::Yes) psOut->setScale(xScale, yScale);
         }
 
+        QValueList<int> pageList;
+
         if (!printer.previewOnly())
         {
-            QValueList<int> pageList = printer.pageList();
-            QValueList<int>::const_iterator it;
-
-            for(it = pageList.begin(); it != pageList.end(); ++it) pages.push_back(*it);
+            pageList = printer.pageList();
         }
         else
         {
-            for(int i = 1; i <= pdfdoc->getNumPages(); i++) pages.push_back(i);
+            for(int i = 1; i <= pdfdoc->getNumPages(); i++) pageList.append(i);
         }
 
+        QValueList<int>::const_iterator pIt = pageList.begin(), pEnd = pageList.end();
         docLock.lock();
-        pdfdoc->displayPages(psOut, pages, 72, 72, 0, false, globalParams->getPSCrop(), gFalse);
+        for ( ; pIt != pEnd; ++pIt )
+        {
+            pdfdoc->displayPage(psOut, *pIt, 72, 72, 0, false, globalParams->getPSCrop(), gFalse);
+        }
         docLock.unlock();
 
         // needs to be here so that the file is flushed, do not merge with the one
@@ -494,12 +515,53 @@ bool PDFGenerator::print( KPrinter& printer )
     }
 }
 
-static UGString *QStringToUGString(const QString &s) {
+static GString *QStringToGString(const QString &s) {
     int len = s.length();
-    Unicode *u = (Unicode *)gmallocn(s.length(), sizeof(Unicode));
+    char *cstring = (char *)gmallocn(s.length(), sizeof(char));
     for (int i = 0; i < len; ++i)
-      u[i] = s.at(i).unicode();
-    return new UGString(u, len);
+      cstring[i] = s.at(i).unicode();
+    return new GString(cstring, len);
+}
+
+static QString unicodeToQString(Unicode* u, int len) {
+    QString ret;
+    ret.setLength(len);
+    QChar* qch = (QChar*) ret.unicode();
+    for (;len;--len)
+      *qch++ = (QChar) *u++;
+    return ret;
+}
+
+static QString UnicodeParsedString(GString *s1) {
+    GBool isUnicode;
+    int i;
+    Unicode u;
+    QString result;
+    if ( ( s1->getChar(0) & 0xff ) == 0xfe && ( s1->getChar(1) & 0xff ) == 0xff )
+    {
+        isUnicode = gTrue;
+        i = 2;
+    }
+    else
+    {
+        isUnicode = gFalse;
+        i = 0;
+    }
+    while ( i < s1->getLength() )
+    {
+        if ( isUnicode )
+        {
+            u = ( ( s1->getChar(i) & 0xff ) << 8 ) | ( s1->getChar(i+1) & 0xff );
+            i += 2;
+        }
+        else
+        {
+            u = s1->getChar(i) & 0xff;
+            ++i;
+        }
+        result += unicodeToQString( &u, 1 );
+    }
+    return result;
 }
 
 QString PDFGenerator::getMetaData( const QString & key, const QString & option )
@@ -515,7 +577,7 @@ QString PDFGenerator::getMetaData( const QString & key, const QString & option )
         // asking for the page related to a 'named link destination'. the
         // option is the link name. @see addSynopsisChildren.
         DocumentViewport viewport;
-        UGString * namedDest = QStringToUGString( option );
+        GString * namedDest = QStringToGString(option);
         docLock.lock();
         LinkDest * destination = pdfdoc->findDest( namedDest );
         if ( destination )
@@ -527,6 +589,11 @@ QString PDFGenerator::getMetaData( const QString & key, const QString & option )
         if ( viewport.pageNumber >= 0 )
             return viewport.toString();
     }
+    else if ( key == "OpenTOC" )
+    {
+        if ( pdfdoc->getCatalog()->getPageMode() == Catalog::UseOutlines )
+            return "yes";
+    }
     return QString();
 }
 
@@ -558,15 +625,6 @@ bool PDFGenerator::reparseConfig()
 }
 //END Generator inherited functions
 
-static QString unicodeToQString(Unicode* u, int len) {
-    QString ret;
-    ret.setLength(len);
-    QChar* qch = (QChar*) ret.unicode();
-    for (;len;--len)
-      *qch++ = (QChar) *u++;
-    return ret;
-}
-
 void PDFGenerator::scanFonts(Dict *resDict, KListView *list, Ref **fonts, int &fontsLen, int &fontsSize)
 {
     Object obj1, obj2, xObjDict, xObj, resObj;
@@ -630,15 +688,19 @@ void PDFGenerator::scanFont(GfxFont *font, KListView *list, Ref **fonts, int &fo
     GBool emb;
     int i;
 
-    QString fontTypeNames[8] = {
+    QString fontTypeNames[12] = {
         i18n("unknown"),
         i18n("Type 1"),
         i18n("Type 1C"),
+        i18n("OT means OpenType", "Type 1C (OT)"),
         i18n("Type 3"),
         i18n("TrueType"),
+        i18n("OT means OpenType", "TrueType (OT)"),
         i18n("CID Type 0"),
         i18n("CID Type 0C"),
-        i18n("CID TrueType")
+        i18n("OT means OpenType", "CID Type 0C (OT)"),
+        i18n("CID TrueType"),
+        i18n("OT means OpenType", "CID TrueType (OT)")
     };
 
     fontRef = *font->getID();
@@ -704,41 +766,12 @@ QString PDFGenerator::getDocumentInfo( const QString & data, bool canReturnNull
     if ( !info.isDict() )
         return canReturnNull ? QString::null : i18n( "Unknown" );
 
-    QString result;
     Object obj;
-    GString *s1;
-    GBool isUnicode;
-    Unicode u;
-    int i;
     Dict *infoDict = info.getDict();
 
-    if ( infoDict->lookup( data.latin1(), &obj )->isString() )
+    if ( infoDict->lookup( (char*)data.latin1(), &obj )->isString() )
     {
-        s1 = obj.getString();
-        if ( ( s1->getChar(0) & 0xff ) == 0xfe && ( s1->getChar(1) & 0xff ) == 0xff )
-        {
-            isUnicode = gTrue;
-            i = 2;
-        }
-        else
-        {
-            isUnicode = gFalse;
-            i = 0;
-        }
-        while ( i < obj.getString()->getLength() )
-        {
-            if ( isUnicode )
-            {
-                u = ( ( s1->getChar(i) & 0xff ) << 8 ) | ( s1->getChar(i+1) & 0xff );
-                i += 2;
-            }
-            else
-            {
-                u = s1->getChar(i) & 0xff;
-                ++i;
-            }
-            result += unicodeToQString( &u, 1 );
-        }
+        QString result = UnicodeParsedString(obj.getString());
         obj.free();
         info.free();
         return result;
@@ -761,7 +794,6 @@ QString PDFGenerator::getDocumentDate( const QString & data ) const
         return i18n( "Unknown Date" );
 
     Object obj;
-    const char *s, *origS;
     int year, mon, day, hour, min, sec;
     Dict *infoDict = info.getDict();
     UnicodeMap *uMap = globalParams->getTextEncoding();
@@ -770,14 +802,13 @@ QString PDFGenerator::getDocumentDate( const QString & data ) const
     if ( !uMap )
         return i18n( "Unknown Date" );
 
-    if ( infoDict->lookup( data.latin1(), &obj )->isString() )
+    if ( infoDict->lookup( (char*)data.latin1(), &obj )->isString() )
     {
-        s = UGString(*obj.getString()).getCString();
-        origS = s;
+        QString s = UnicodeParsedString(obj.getString());
         if ( s[0] == 'D' && s[1] == ':' )
-            s += 2;
+            s = s.mid(2);
 
-        if ( sscanf( s, "%4d%2d%2d%2d%2d%2d", &year, &mon, &day, &hour, &min, &sec ) == 6 )
+        if ( sscanf( s.latin1(), "%4d%2d%2d%2d%2d%2d", &year, &mon, &day, &hour, &min, &sec ) == 6 )
         {
             QDate d( year, mon, day );  //CHECK: it was mon-1, Jan->0 (??)
             QTime t( hour, min, sec );
@@ -788,7 +819,6 @@ QString PDFGenerator::getDocumentDate( const QString & data ) const
         }
         else
             result = s;
-        delete[] origS;
     }
     else
         result = i18n( "Unknown Date" );
@@ -828,9 +858,12 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, GList * items )
                 // get the destination for the page now, but it's VERY time consuming,
                 // so better storing the reference and provide the viewport as metadata
                 // on demand
-                UGString *s = g->getNamedDest();
-                QString aux = unicodeToQString( s->unicode(), s->getLength() );
-                item.setAttribute( "ViewportName", aux );
+                GString *s = g->getNamedDest();
+                QChar *charArray = new QChar[s->getLength()];
+                for (int i = 0; i < s->getLength(); ++i) charArray[i] = QChar(s->getCString()[i]);
+                QString option(charArray, s->getLength());
+                item.setAttribute( "ViewportName", option );
+                delete[] charArray;
             }
             else if ( destination && destination->isOk() )
             {
@@ -845,6 +878,8 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, GList * items )
             }
         }
 
+        item.setAttribute( "Open", QVariant( (bool)outlineItem->isOpen() ).toString() );
+
         // 3. recursively descend over children
         outlineItem->open();
         GList * children = outlineItem->getKids();
@@ -876,7 +911,7 @@ void PDFGenerator::fillViewportFromLink( DocumentViewport &viewport, LinkDest *d
                 double CTM[6];
                 Page *page = pdfdoc->getCatalog()->getPage( viewport.pageNumber + 1 );
                 // TODO remember to change this if we implement DPI and/or rotation
-                page->getDefaultCTM(CTM, 72.0, 72.0, 0, gTrue);
+                page->getDefaultCTM(CTM, 72.0, 72.0, 0, gFalse, gTrue);
 
                 int left, top;
                 // this is OutputDev::cvtUserToDev
@@ -1163,7 +1198,8 @@ void PDFPixmapGeneratorThread::run()
                                             genObjectRects, genObjectRects, TRUE /*thread safety*/ );
     d->generator->pdfdoc->displayPage( d->generator->kpdfOutputDev, page->number() + 1,
                                        fakeDpiX, fakeDpiY, 0, false, true, genObjectRects );
-    delete d->generator->pdfdoc->takeLinks(); // we have already created the KPDFLinks so free the xpdf memory
+    if ( genObjectRects )
+        d->generator->pdfdoc->processLinks( d->generator->kpdfOutputDev, page->number() + 1 );
 
     // 2. grab data from the OutputDev and store it locally (note takeIMAGE)
 #ifndef NDEBUG
