@@ -118,8 +118,10 @@ protected slots:
 	void enableTOC(bool enable);
 	void psTransformEnded();
 	void cannotQuit();
-  void saveSplitterSize();
+	void saveSplitterSize();
 	void setMimeTypes(KIO::Job *job);
+	void readMimeType(KIO::Job *job, const QString &mime);
+	void emitWindowCaption();
 
 public slots:
 	// connected to Shell action (and browserExtension), not local one
@@ -159,9 +161,13 @@ private:
 	QTimer *m_dirtyHandler;
 	DocumentViewport m_viewportDirty;
 	bool m_wasPresentationOpen;
+	int m_dirtyToolboxIndex;
 	
 	// Remember the search history
 	QStringList m_searchHistory;
+	
+	// mimetype got from the job
+	QString m_jobMime;
 
 	// actions
 	KAction *m_gotoPage;
