@@ -46,7 +46,7 @@ class PresentationWidget : public QDialog, public DocumentObserver
 
     protected:
         // widget events
-        bool event( QEvent * e );
+//        bool event( QEvent * e );
         void keyPressEvent( QKeyEvent * e );
         void wheelEvent( QWheelEvent * e );
         void mousePressEvent( QMouseEvent * e );
@@ -79,6 +79,7 @@ class PresentationWidget : public QDialog, public DocumentObserver
         // transition related
         QTimer * m_transitionTimer;
         QTimer * m_overlayHideTimer;
+        QTimer * m_nextPageTimer;
         int m_transitionDelay;
         int m_transitionMul;
         QValueList< QRect > m_transitionRects;
