@@ -7,6 +7,7 @@
 #include <aconf.h>
 
 #if HAVE_FREETYPE_FREETYPE_H || HAVE_FREETYPE_H
+
 #ifdef USE_GCC_PRAGMAS
 #pragma implementation
 #endif
@@ -30,20 +31,26 @@ extern "C" int unlink(char *filename);
 #endif
 
 //------------------------------------------------------------------------
-
-#if 0
-static void FT_fileWrite(void *stream, const char *data, int len) {
+/*
+static void fileWrite(void *stream, char *data, int len) {
   fwrite(data, 1, len, (FILE *)stream);
 }
-#endif
+*/
 
 //------------------------------------------------------------------------
 // SplashFTFontEngine
 //------------------------------------------------------------------------
 
 SplashFTFontEngine::SplashFTFontEngine(GBool aaA, FT_Library libA) {
+  FT_Int major, minor, patch;
+
   aa = aaA;
   lib = libA;
+
+  // as of FT 2.1.8, CID fonts are indexed by CID instead of GID
+  FT_Library_Version(lib, &major, &minor, &patch);
+  useCIDs = major > 2 ||
+            (major == 2 && (minor > 1 || (minor == 1 && patch > 7)));
 }
 
 SplashFTFontEngine *SplashFTFontEngine::init(GBool aaA) {
@@ -61,43 +68,80 @@ SplashFTFontEngine::~SplashFTFontEngine() {
 
 SplashFontFile *SplashFTFontEngine::loadType1Font(SplashFontFileID *idA,
 						  SplashFontSrc *src,
-						  const char **enc) {
+						  char **enc) {
   return SplashFTFontFile::loadType1Font(this, idA, src, enc);
 }
 
 SplashFontFile *SplashFTFontEngine::loadType1CFont(SplashFontFileID *idA,
 						   SplashFontSrc *src,
-						   const char **enc) {
+						   char **enc) {
+  return SplashFTFontFile::loadType1Font(this, idA, src, enc);
+}
+
+SplashFontFile *SplashFTFontEngine::loadOpenTypeT1CFont(SplashFontFileID *idA,
+							SplashFontSrc *src,
+							char **enc) {
   return SplashFTFontFile::loadType1Font(this, idA, src, enc);
 }
 
 SplashFontFile *SplashFTFontEngine::loadCIDFont(SplashFontFileID *idA,
 						SplashFontSrc *src) {
+  FoFiType1C *ff;
   Gushort *cidToGIDMap;
   int nCIDs;
   SplashFontFile *ret;
 
   // check for a CFF font
-#if HAVE_FREETYPE_217_OR_OLDER
-  FoFiType1C *ff;
-  if (src->isFile) {
-    ff = FoFiType1C::load(src->fileName->getCString());
-  } else {
-    ff = new FoFiType1C(src->buf, src->bufLen, gFalse);
-  }
-  if (ff) {
-    cidToGIDMap = ff->getCIDToGIDMap(&nCIDs);
-    delete ff;
-  } else {
+  if (useCIDs) {
     cidToGIDMap = NULL;
     nCIDs = 0;
+  } else {
+    if (src->isFile) {
+      ff = FoFiType1C::load(src->fileName->getCString());
+    } else {
+      ff = FoFiType1C::make(src->buf, src->bufLen);
+    }
+    if (ff) {
+      cidToGIDMap = ff->getCIDToGIDMap(&nCIDs);
+      delete ff;
+    } else {
+      cidToGIDMap = NULL;
+      nCIDs = 0;
+    }
   }
-#else
-  // Freetype 2.1.8 and up treats all CID fonts the same way
+  ret = SplashFTFontFile::loadCIDFont(this, idA, src, cidToGIDMap, nCIDs);
+  if (!ret) {
+    gfree(cidToGIDMap);
+  }
+  return ret;
+}
+
+SplashFontFile *SplashFTFontEngine::loadOpenTypeCFFFont(SplashFontFileID *idA,
+							SplashFontSrc *src) {
+  FoFiTrueType *ff;
+  GBool isCID;
+  Gushort *cidToGIDMap;
+  int nCIDs;
+  SplashFontFile *ret;
+
   cidToGIDMap = NULL;
   nCIDs = 0;
-#endif
-  ret = SplashFTFontFile::loadCIDFont(this, idA, src, cidToGIDMap, nCIDs);
+  isCID = gFalse;
+  if (!useCIDs) {
+    if (src->isFile) {
+      ff = FoFiTrueType::load(src->fileName->getCString());
+    } else {
+      ff = FoFiTrueType::make(src->buf, src->bufLen);
+    }
+    if (ff) {
+      if (ff->isOpenTypeCFF()) {
+	cidToGIDMap = ff->getCIDToGIDMap(&nCIDs);
+      }
+      delete ff;
+    }
+  }
+  ret = SplashFTFontFile::loadCIDFont(this, idA, src,
+				      cidToGIDMap, nCIDs);
   if (!ret) {
     gfree(cidToGIDMap);
   }
@@ -115,7 +159,7 @@ SplashFontFile *SplashFTFontEngine::loadTrueTypeFont(SplashFontFileID *idA,
   FILE *tmpFile;
   SplashFontFile *ret;
 
-  if (!(ff = FoFiTrueType::load(fileName, faceIndex))) {
+  if (!(ff = FoFiTrueType::load(fileName))) {
     return NULL;
   }
   tmpFileName = NULL;
@@ -123,13 +167,12 @@ SplashFontFile *SplashFTFontEngine::loadTrueTypeFont(SplashFontFileID *idA,
     delete ff;
     return NULL;
   }
-  ff->writeTTF(&FT_fileWrite, tmpFile);
+  ff->writeTTF(&fileWrite, tmpFile);
   delete ff;
   fclose(tmpFile);
   ret = SplashFTFontFile::loadTrueTypeFont(this, idA,
 					   tmpFileName->getCString(),
-					   gTrue, codeToGID, codeToGIDLen,
-					   faceIndex);
+					   gTrue, codeToGID, codeToGIDLen);
   if (ret) {
     if (deleteFile) {
       unlink(fileName);
