@@ -27,6 +27,7 @@ class GfxFont;
 class GfxColorSpace;
 class GfxSeparationColorSpace;
 class PDFRectangle;
+struct PSFont8Info;
 struct PSFont16Enc;
 class PSOutCustomColor;
 
@@ -47,24 +48,17 @@ enum PSFileType {
   psGeneric			// write to a generic stream
 };
 
-typedef void (*PSOutputFunc)(void *stream, const char *data, int len);
+typedef void (*PSOutputFunc)(void *stream, char *data, int len);
 
 class PSOutputDev: public OutputDev {
 public:
 
   // Open a PostScript output file, and write the prolog.
-  PSOutputDev(const char *fileName, XRef *xrefA, Catalog *catalog,
-	      int firstPage, int lastPage, PSOutMode modeA,
-	      int imgLLXA = 0, int imgLLYA = 0,
-	      int imgURXA = 0, int imgURYA = 0,
-	      GBool manualCtrlA = gFalse);
-
-  // Open a PSOutputDev that will write to a generic stream.
-  PSOutputDev(PSOutputFunc outputFuncA, void *outputStreamA,
-	      XRef *xrefA, Catalog *catalog,
+  PSOutputDev(char *fileName, char *pstitle, XRef *xrefA, Catalog *catalog,
 	      int firstPage, int lastPage, PSOutMode modeA,
 	      int imgLLXA = 0, int imgLLYA = 0,
 	      int imgURXA = 0, int imgURYA = 0,
+	      GBool forceRasterize = gFalse,
 	      GBool manualCtrlA = gFalse);
 
   // Destructor -- writes the trailer and closes the file.
@@ -91,7 +85,11 @@ public:
   // radialShadedFill()?  If this returns false, these shaded fills
   // will be reduced to a series of other drawing operations.
   virtual GBool useShadedFills()
-    { return level == psLevel2 || level == psLevel3; }
+    { return level >= psLevel2; }
+
+  // Does this device use drawForm()?  If this returns false,
+  // form-type XObjects will be interpreted (i.e., unrolled).
+  virtual GBool useDrawForm() { return preload; }
 
   // Does this device use beginType3Char/endType3Char?  Otherwise,
   // text in Type 3 fonts will be drawn with drawChar/drawString.
@@ -102,7 +100,7 @@ public:
   // Write the document-level header.
   void writeHeader(int firstPage, int lastPage,
 		   PDFRectangle *mediaBox, PDFRectangle *cropBox,
-		   int pageRotate);
+		   int pageRotate, char *pstitle);
 
   // Write the Xpdf procset.
   void writeXpdfProcset();
@@ -118,6 +116,17 @@ public:
 
   //----- initialization and control
 
+  // Check to see if a page slice should be displayed.  If this
+  // returns false, the page display is aborted.  Typically, an
+  // OutputDev will use some alternate means to display the page
+  // before returning false.
+  virtual GBool checkPageSlice(Page *page, double hDPI, double vDPI,
+			       int rotate, GBool useMediaBox, GBool crop,
+			       int sliceX, int sliceY, int sliceW, int sliceH,
+			       GBool printing, Catalog *catalog,
+			       GBool (*abortCheckCbk)(void *data) = NULL,
+			       void *abortCheckCbkData = NULL);
+
   // Start a page.
   virtual void startPage(int pageNum, GfxState *state);
 
@@ -143,6 +152,7 @@ public:
   virtual void updateStrokeColor(GfxState *state);
   virtual void updateFillOverprint(GfxState *state);
   virtual void updateStrokeOverprint(GfxState *state);
+  virtual void updateTransfer(GfxState *state);
 
   //----- update text state
   virtual void updateFont(GfxState *state);
@@ -164,14 +174,15 @@ public:
 				 double *mat, double *bbox,
 				 int x0, int y0, int x1, int y1,
 				 double xStep, double yStep);
-  virtual void functionShadedFill(GfxState *state,
-				  GfxFunctionShading *shading);
-  virtual void axialShadedFill(GfxState *state, GfxAxialShading *shading);
-  virtual void radialShadedFill(GfxState *state, GfxRadialShading *shading);
+  virtual GBool functionShadedFill(GfxState *state,
+				   GfxFunctionShading *shading);
+  virtual GBool axialShadedFill(GfxState *state, GfxAxialShading *shading);
+  virtual GBool radialShadedFill(GfxState *state, GfxRadialShading *shading);
 
   //----- path clipping
   virtual void clip(GfxState *state);
   virtual void eoClip(GfxState *state);
+  virtual void clipToStrokePath(GfxState *state);
 
   //----- text drawing
   virtual void drawString(GfxState *state, GString *s);
@@ -201,6 +212,9 @@ public:
   virtual void type3D1(GfxState *state, double wx, double wy,
 		       double llx, double lly, double urx, double ury);
 
+  //----- form XObjects
+  virtual void drawForm(Ref ref);
+
   //----- PostScript XObjects
   virtual void psXObject(Stream *psStream, Stream *level1Stream);
 
@@ -223,7 +237,7 @@ public:
 private:
 
   void init(PSOutputFunc outputFuncA, void *outputStreamA,
-	    PSFileType fileTypeA, XRef *xrefA, Catalog *catalog,
+	    PSFileType fileTypeA, char *pstitle, XRef *xrefA, Catalog *catalog,
 	    int firstPage, int lastPage, PSOutMode modeA,
 	    int imgLLXA, int imgLLYA, int imgURXA, int imgURYA,
 	    GBool manualCtrlA);
@@ -233,15 +247,19 @@ private:
   void setupEmbeddedType1Font(Ref *id, GString *psName);
   void setupExternalType1Font(GString *fileName, GString *psName);
   void setupEmbeddedType1CFont(GfxFont *font, Ref *id, GString *psName);
+  void setupEmbeddedOpenTypeT1CFont(GfxFont *font, Ref *id, GString *psName);
   void setupEmbeddedTrueTypeFont(GfxFont *font, Ref *id, GString *psName);
   GString *setupExternalTrueTypeFont(GfxFont *font);
   void setupEmbeddedCIDType0Font(GfxFont *font, Ref *id, GString *psName);
   void setupEmbeddedCIDTrueTypeFont(GfxFont *font, Ref *id, GString *psName,
-				   GBool needsVerticalMetrics);
+				    GBool needVerticalMetrics);
+  void setupEmbeddedOpenTypeCFFFont(GfxFont *font, Ref *id, GString *psName);
   GString *setupExternalCIDTrueTypeFont(GfxFont *font, GString *fileName, int faceIndex=0);
   void setupType3Font(GfxFont *font, GString *psName, Dict *parentResDict);
   void setupImages(Dict *resDict);
   void setupImage(Ref id, Stream *str);
+  void setupForms(Dict *resDict);
+  void setupForm(Ref id, Object *strObj);
   void addProcessColor(double c, double m, double y, double k);
   void addCustomColor(GfxSeparationColorSpace *sepCS);
   void doPath(GfxPath *path);
@@ -256,8 +274,14 @@ private:
 		 Stream *str, int width, int height, int len,
 		 int *maskColors, Stream *maskStr,
 		 int maskWidth, int maskHeight, GBool maskInvert);
+  void doImageL3(Object *ref, GfxImageColorMap *colorMap,
+		 GBool invert, GBool inlineImg,
+		 Stream *str, int width, int height, int len,
+		 int *maskColors, Stream *maskStr,
+		 int maskWidth, int maskHeight, GBool maskInvert);
   void dumpColorSpaceL2(GfxColorSpace *colorSpace,
-			GBool genXform, GBool updateColors);
+			GBool genXform, GBool updateColors,
+			GBool map01);
 #if OPI_SUPPORT
   void opiBegin20(GfxState *state, Dict *dict);
   void opiBegin13(GfxState *state, Dict *dict);
@@ -267,11 +291,12 @@ private:
 #endif
   void cvtFunction(Function *func);
   void writePSChar(char c);
-  void writePS(const char *s);
+  void writePS(char *s);
   void writePSFmt(const char *fmt, ...);
   void writePSString(GString *s);
-  void writePSName(const char *s);
+  void writePSName(char *s);
   GString *filterPSName(GString *name);
+  void writePSTextLine(GString *s);
 
   PSLevel level;		// PostScript level (1, 2, separation)
   PSOutMode mode;		// PostScript mode (PS, EPS, form)
@@ -279,6 +304,8 @@ private:
   int paperHeight;		// height of paper, in pts
   int imgLLX, imgLLY,		// imageable area, in pts
       imgURX, imgURY;
+  GBool preload;		// load all images into memory, and
+				//   predefine forms
 
   PSOutputFunc outputFunc;
   void *outputStream;
@@ -304,9 +331,18 @@ private:
   int fontFileNameSize;		// size of fontFileNames array
   int nextTrueTypeNum;		// next unique number to append to a TrueType
 				//   font name
+  PSFont8Info *font8Info;	// info for 8-bit fonts
+  int font8InfoLen;		// number of entries in font8Info array
+  int font8InfoSize;		// size of font8Info array
   PSFont16Enc *font16Enc;	// encodings for substitute 16-bit fonts
   int font16EncLen;		// number of entries in font16Enc array
   int font16EncSize;		// size of font16Enc array
+  Ref *imgIDs;			// list of image IDs for in-memory images
+  int imgIDLen;			// number of entries in imgIDs array
+  int imgIDSize;		// size of imgIDs array
+  Ref *formIDs;			// list of IDs for predefined forms
+  int formIDLen;		// number of entries in formIDs array
+  int formIDSize;		// size of formIDs array
   GList *xobjStack;		// stack of XObject dicts currently being
 				//   processed
   int numSaves;			// current number of gsaves
@@ -338,6 +374,8 @@ private:
   double t3WX, t3WY,		// Type 3 character parameters
          t3LLX, t3LLY, t3URX, t3URY;
   GBool t3Cacheable;		// cleared if char is not cacheable
+  GBool t3NeedsRestore;		// set if a 'q' operator was issued
+  GBool forceRasterize;		// forces the page to be rasterized
 
 #if OPI_SUPPORT
   int opi13Nest;		// nesting level of OPI 1.3 objects
