@@ -28,7 +28,7 @@
 
 #include <klocale.h>
 
-PMDefinePropertyClass( PMBlobCylinder, PMProperty );
+PMDefinePropertyClass( PMBlobCylinder, PMBlobCylinderProperty );
 
 PMMetaObject* PMBlobCylinder::s_pMetaObject = 0;
 PMObject* createNewBlobCylinder( PMPart* part )
@@ -100,16 +100,16 @@ PMMetaObject* PMBlobCylinder::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "BlobCylinder", Base::metaObject( ),
                                         createNewBlobCylinder );
       s_pMetaObject->addProperty(
-         new PMProperty( "end1", &PMBlobCylinder::setEnd1,
+         new PMBlobCylinderProperty( "end1", &PMBlobCylinder::setEnd1,
                          &PMBlobCylinder::end1 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "end2", &PMBlobCylinder::setEnd2,
+         new PMBlobCylinderProperty( "end2", &PMBlobCylinder::setEnd2,
                          &PMBlobCylinder::end2 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "radius", &PMBlobCylinder::setRadius,
+         new PMBlobCylinderProperty( "radius", &PMBlobCylinder::setRadius,
                          &PMBlobCylinder::radius ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "strength", &PMBlobCylinder::setStrength,
+         new PMBlobCylinderProperty( "strength", &PMBlobCylinder::setStrength,
                          &PMBlobCylinder::strength ) );
    }
    return s_pMetaObject;
@@ -201,7 +201,8 @@ void PMBlobCylinder::restoreMemento( PMMemento* s )
 bool PMBlobCylinder::isDefault( )
 {
    if( ( m_end1 == c_defaultEnd1 ) && ( m_end2 == c_defaultEnd2 )
-       && ( m_radius == c_defaultRadius ) )
+       && ( m_radius == c_defaultRadius )
+       && globalDetail( ) )
       return true;
    return false;
 }
@@ -214,69 +215,89 @@ void PMBlobCylinder::createViewStructure( )
       m_pViewStructure->points( ).detach( );
    }
 
-   createPoints( m_pViewStructure->points( ), m_end1, m_end2, m_radius );
+   int uStep = (int)( ( (float)s_uStep / 2 ) * ( displayDetail( ) + 1 ) );
+   int vStep = (int)( ( (float)s_vStep / 2 ) * ( displayDetail( ) + 1 ) );
+   unsigned ptsSize = vStep * uStep * 2 + 2;
+   unsigned lineSize = vStep * uStep * 4 + vStep;
+
+   if( ptsSize != m_pViewStructure->points( ).size( ) )
+      m_pViewStructure->points( ).resize( ptsSize );
+
+   createPoints( m_pViewStructure->points( ), m_end1, m_end2, m_radius, uStep, vStep );
+
+   if( lineSize != m_pViewStructure->lines( ).size( ) )
+   {
+      m_pViewStructure->lines( ).detach( );
+      m_pViewStructure->lines( ).resize( lineSize );
+      createLines( m_pViewStructure->lines( ), uStep, vStep );
+   }
 }
 
 PMViewStructure* PMBlobCylinder::defaultViewStructure( ) const
 {
-   if( !s_pDefaultViewStructure )
+   if( !s_pDefaultViewStructure || s_pDefaultViewStructure->parameterKey( ) != viewStructureParameterKey( ) )
    {
-      int u, v;
+      delete s_pDefaultViewStructure;
+      s_pDefaultViewStructure = 0;
+      int uStep = (int)( ( (float)s_uStep / 2 ) * ( displayDetail( ) + 1 ) );
+      int vStep = (int)( ( (float)s_vStep / 2 ) * ( displayDetail( ) + 1 ) );
 
       s_pDefaultViewStructure =
-         new PMViewStructure( s_vStep * s_uStep * 2 + 2,
-                              s_vStep * s_uStep * 4 + s_vStep );
+         new PMViewStructure( vStep * uStep * 2 + 2,
+                              vStep * uStep * 4 + vStep );
 
-      PMLineArray& lines = s_pDefaultViewStructure->lines( );
-
-
-      // points
       createPoints( s_pDefaultViewStructure->points( ), c_defaultEnd1,
-                    c_defaultEnd2, c_defaultRadius );
+                    c_defaultEnd2, c_defaultRadius, uStep, vStep );
 
-      int offset = 0;
-
-      // horizontal lines
-      for( u = 0; u < ( s_uStep * 2 ); u++ )
-      {
-         for( v = 0; v < ( s_vStep - 1 ); v++ )
-            lines[offset + v] =
-               PMLine( u * s_vStep + v + 1, u * s_vStep + v + 2 );
-         lines[offset + s_vStep - 1] =
-            PMLine( u * s_vStep + 1, u * s_vStep + s_vStep );
-
-         offset += s_vStep;
-      }
-
-      // vertical lines
-      // lines at the "north pole"
-      for( v = 0; v < s_vStep; v++ )
-         lines[offset + v] = PMLine( 0, v + 1 );
-      offset += s_vStep;
-
-      for( v = 0; v < s_vStep; v++ )
-      {
-         for( u = 0; u < ( 2 * s_uStep - 1 ); u++ )
-         {
-            lines[offset + u] =
-               PMLine( u * s_vStep + v + 1, ( u + 1 ) * s_vStep + v + 1 );
-         }
-         offset += ( 2 * s_uStep - 1 );
-      }
-      // lines at the "south pole"
-      for( v = 0; v < s_vStep; v++ )
-         lines[offset + v] = PMLine( ( 2 * s_uStep - 1 ) * s_vStep + v + 1,
-                                     2 * s_uStep * s_vStep + 1 );
-      // offset += s_vStep;
+      createLines( s_pDefaultViewStructure->lines( ), uStep, vStep );
    }
    return s_pDefaultViewStructure;
 }
 
-void PMBlobCylinder::createPoints( PMPointArray& points, const PMVector& end1,
-                                   const PMVector& end2, double radius )
+void PMBlobCylinder::createLines( PMLineArray& lines, int uStep, int vStep )
 {
-   double uRadStep = M_PI / s_uStep / 2.0;
-   double vRadStep = 2 * M_PI / s_vStep;
+   int u, v;
+   int offset = 0;
+
+   // horizontal lines
+   for( u = 0; u < ( uStep * 2 ); u++ )
+   {
+      for( v = 0; v < ( vStep - 1 ); v++ )
+         lines[offset + v] =
+            PMLine( u * vStep + v + 1, u * vStep + v + 2 );
+      lines[offset + vStep - 1] =
+         PMLine( u * vStep + 1, u * vStep + vStep );
+
+      offset += vStep;
+   }
+
+   // vertical lines
+   // lines at the "north pole"
+   for( v = 0; v < vStep; v++ )
+      lines[offset + v] = PMLine( 0, v + 1 );
+   offset += vStep;
+
+   for( v = 0; v < vStep; v++ )
+   {
+      for( u = 0; u < ( 2 * uStep - 1 ); u++ )
+      {
+         lines[offset + u] =
+            PMLine( u * vStep + v + 1, ( u + 1 ) * vStep + v + 1 );
+      }
+      offset += ( 2 * uStep - 1 );
+   }
+   // lines at the "south pole"
+   for( v = 0; v < vStep; v++ )
+      lines[offset + v] = PMLine( ( 2 * uStep - 1 ) * vStep + v + 1,
+                                    2 * uStep * vStep + 1 );
+   // offset += vStep;
+}
+
+void PMBlobCylinder::createPoints( PMPointArray& points, const PMVector& end1,
+                                   const PMVector& end2, double radius, int uStep, int vStep )
+{
+   double uRadStep = M_PI / uStep / 2.0;
+   double vRadStep = 2 * M_PI / vStep;
    double du = uRadStep;
 
    if( radius < 0 )
@@ -294,20 +315,20 @@ void PMBlobCylinder::createPoints( PMPointArray& points, const PMVector& end1,
    ortho /= ortho.abs( );
 
    points[0] = PMPoint( end1 - pointAt * radius );
-   points[s_vStep * s_uStep * 2 + 1] = PMPoint( end2 + pointAt * radius );
+   points[vStep * uStep * 2 + 1] = PMPoint( end2 + pointAt * radius );
 
    int u, v;
-   for( u = 0; u < s_uStep; u++ )
+   for( u = 0; u < uStep; u++ )
    {
       PMVector end = ortho * radius * sin( du );
       PMVector pv = pointAt * radius * cos( du );
       PMVector e1 = end1 - pv;
       PMVector e2 = end2 + pv;
 
-      for( v = 0; v < s_vStep; v++ )
+      for( v = 0; v < vStep; v++ )
       {
-         points[u * s_vStep + v + 1] = PMPoint( e1 + end );
-         points[s_vStep * s_uStep * 2 - ( u + 1 ) * s_vStep + v + 1]
+         points[u * vStep + v + 1] = PMPoint( e1 + end );
+         points[vStep * uStep * 2 - ( u + 1 ) * vStep + v + 1]
             = PMPoint( e2 + end );
          end = rotation * end;
       }
