@@ -30,7 +30,7 @@ const double defaultBoxSize = 0.5;
 const PMVector corner1Default = PMVector( -defaultBoxSize, -defaultBoxSize, -defaultBoxSize );
 const PMVector corner2Default = PMVector( defaultBoxSize, defaultBoxSize, defaultBoxSize );
 
-PMDefinePropertyClass( PMBox, PMProperty );
+PMDefinePropertyClass( PMBox, PMBoxProperty );
 
 PMViewStructure* PMBox::s_pDefaultViewStructure = 0;
 PMMetaObject* PMBox::s_pMetaObject = 0;
@@ -83,9 +83,9 @@ PMMetaObject* PMBox::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Box", Base::metaObject( ),
                                         createNewBox );
       s_pMetaObject->addProperty(
-         new PMProperty( "corner1", &PMBox::setCorner1, &PMBox::corner1 ) );
+         new PMBoxProperty( "corner1", &PMBox::setCorner1, &PMBox::corner1 ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "corner2", &PMBox::setCorner2, &PMBox::corner2 ) );
+         new PMBoxProperty( "corner2", &PMBox::setCorner2, &PMBox::corner2 ) );
    }
    return s_pMetaObject;
 }
