@@ -28,7 +28,7 @@
 
 #include <klocale.h>
 
-PMDefinePropertyClass( PMCamera, PMProperty );
+PMDefinePropertyClass( PMCamera, PMCameraProperty );
 PMDefineEnumPropertyClass( PMCamera, PMCamera::CameraType,
                            PMCameraTypeProperty );
 
@@ -172,31 +172,31 @@ PMMetaObject* PMCamera::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Camera", Base::metaObject( ),
                                         createNewCamera );
       s_pMetaObject->addProperty(
-         new PMProperty( "location", &PMCamera::setLocation,
+         new PMCameraProperty( "location", &PMCamera::setLocation,
                          &PMCamera::location ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "lookAt", &PMCamera::setLookAt,
+         new PMCameraProperty( "lookAt", &PMCamera::setLookAt,
                          &PMCamera::lookAt ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "up", &PMCamera::setUp,
+         new PMCameraProperty( "up", &PMCamera::setUp,
                          &PMCamera::up ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "right", &PMCamera::setRight,
+         new PMCameraProperty( "right", &PMCamera::setRight,
                          &PMCamera::right ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "direction", &PMCamera::setDirection,
+         new PMCameraProperty( "direction", &PMCamera::setDirection,
                          &PMCamera::direction ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "sky", &PMCamera::setSky,
+         new PMCameraProperty( "sky", &PMCamera::setSky,
                          &PMCamera::sky ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "aspect", 0, &PMCamera::aspect ) );
+         new PMCameraProperty( "aspect", 0, &PMCamera::aspect ) );
 
       s_pMetaObject->addProperty(
-         new PMProperty( "angleEnabled", &PMCamera::enableAngle,
+         new PMCameraProperty( "angleEnabled", &PMCamera::enableAngle,
                          &PMCamera::isAngleEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "angle", &PMCamera::setAngle,
+         new PMCameraProperty( "angle", &PMCamera::setAngle,
                          &PMCamera::angle ) );
 
       PMCameraTypeProperty* p = new PMCameraTypeProperty(
@@ -212,28 +212,28 @@ PMMetaObject* PMCamera::metaObject( ) const
       s_pMetaObject->addProperty( p );
 
       s_pMetaObject->addProperty(
-         new PMProperty( "cylinderType", &PMCamera::setCylinderType,
+         new PMCameraProperty( "cylinderType", &PMCamera::setCylinderType,
                          &PMCamera::cylinderType ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "focalBlurEnabled", &PMCamera::enableFocalBlur,
+         new PMCameraProperty( "focalBlurEnabled", &PMCamera::enableFocalBlur,
                          &PMCamera::isFocalBlurEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "aperture", &PMCamera::setAperture,
+         new PMCameraProperty( "aperture", &PMCamera::setAperture,
                          &PMCamera::aperture ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "blurSamples", &PMCamera::setBlurSamples,
+         new PMCameraProperty( "blurSamples", &PMCamera::setBlurSamples,
                          &PMCamera::blurSamples ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "focalPoint", &PMCamera::setFocalPoint,
+         new PMCameraProperty( "focalPoint", &PMCamera::setFocalPoint,
                          &PMCamera::focalPoint ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "confidence", &PMCamera::setConfidence,
+         new PMCameraProperty( "confidence", &PMCamera::setConfidence,
                          &PMCamera::confidence ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "variance", &PMCamera::setVariance,
+         new PMCameraProperty( "variance", &PMCamera::setVariance,
                          &PMCamera::variance ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "export", &PMCamera::setExportPovray,
+         new PMCameraProperty( "export", &PMCamera::setExportPovray,
                          &PMCamera::exportPovray ) );
    }
    return s_pMetaObject;
