@@ -18,7 +18,7 @@
 
 
 #ifndef PMCSG_H
-#define PMCGS_H
+#define PMCSG_H
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
