@@ -42,7 +42,7 @@ void PMCylinderEdit::createTopWidgets( )
    m_pEnd1 = new PMVectorEdit( "x", "y", "z", this );
    m_pEnd2 = new PMVectorEdit( "x", "y", "z", this );
    m_pRadius = new PMFloatEdit( this );
-   m_pOpen = new QCheckBox( i18n( "Open" ), this );
+   m_pOpen = new QCheckBox( i18n( "type of the object", "Open" ), this );
 
    gl = new QGridLayout( topLayout( ), 2, 2 );
    gl->addWidget( new QLabel( i18n( "End 1:" ), this ), 0, 0 );
