@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 /*
@@ -40,7 +40,7 @@
    - PMDockManager                  - helper class
    - PMDockMainWindow               - IMPORTANT CLASS: a special KMainWindow that can have dockwidgets
    - PMDockArea                     - like PMDockMainWindow but inherits just QWidget
-   
+
    IMPORTANT Note: This file compiles also in Qt-only mode by using the NO_KDE2 precompiler definition!
 */
 
@@ -315,7 +315,7 @@ protected:
    * level widget
    */
   PMDockButton_Private* toDesktopButton;
-   
+
   /**
    * a little button for enabling/disabling dragging the dockwidget with the mouse
    */
@@ -629,7 +629,8 @@ public:
   void setDockWindowTransient (QWidget *parent, bool transientEnabled);
 
   // MODIFICATION (lpassos)
-  /** 
+  /**
+   *
    * Return the current dock position of the widget
    */
   DockPosition getDockPosition( ) const { return currentDockPos; }
@@ -658,7 +659,9 @@ public slots:
    */
    void toDesktop( );
 
-   /** Sets the caption and tab label */
+   /**
+    * Sets the caption and tab label
+    */
    void slotSetCaption( const QString& );
 
 protected:
@@ -671,7 +674,7 @@ protected:
    * @li it is not able to dock to another dockwidget
    */
   void updateHeader();
-  
+
 signals:
   /**
    * Emitted when another dock widget is docking to this.
@@ -707,7 +710,8 @@ signals:
 
 protected slots:
 
-  /** Does several things here when it has noticed that the former brother widget (closest neighbor) gets lost.
+  /**
+   * Does several things here when it has noticed that the former brother widget (closest neighbor) gets lost.
    * The former brother widget is needed for a possible dockback action, to speak with the Beatles:
    * "To get back to where you once belonged" ;-)
    */
@@ -926,7 +930,7 @@ public:
 
   /**
    * Enables opaque resizing. Opaque resizing is initially turned off.
-   * Call this method before you create any dock widgets!   
+   * Call this method before you create any dock widgets!
    */
   void setSplitterOpaqueResize(bool b=true);
 
@@ -960,6 +964,11 @@ public:
    */
   bool splitterHighResolution() const;
 
+  /**
+   * Returns the main dock widget
+   */
+  QWidget* dockMainWidget( ) const { return main; }
+
 signals:
 
   /**
@@ -1042,7 +1051,7 @@ public:
    */
   void findChildDockWidget( const QWidget* p, QWidgetList& l);
   /**
-   * Returns all floating dock widgets 
+   * Returns all floating dock widgets
    */
   void findFloatingWidgets( QPtrList<PMDockWidget>& l );
 
@@ -1397,7 +1406,7 @@ private:
 
 
 #ifdef _JOWENN_EXPERIMENTAL_
-/* Joseph Wenninger jowenn@kde.org Experimental (Just all KMainWindow references changed to QWidget, otherwise nearly exactly the 
+/* Joseph Wenninger jowenn@kde.org Experimental (Just all KMainWindow references changed to QWidget, otherwise nearly exactly the
 same as PMDockMainWindow*/
 
 class EXPORT_DOCKCLASS PMDockArea : public QWidget
