@@ -24,7 +24,7 @@
 #include <config.h>
 #endif
 
-#include "pmnamedobject.h"
+#include "pmdetailobject.h"
 
 
 /**
@@ -33,9 +33,9 @@
  * Objects in povray can be:
  * Finite Solid Primitives, Finite Patch Primitives, Infinite Solid Primitives and Constructive Solid Geometry
  */
-class PMGraphicalObject : public PMNamedObject
+class PMGraphicalObject : public PMDetailObject
 {
-   typedef PMNamedObject Base;
+   typedef PMDetailObject Base;
 public:
    /**
     * Creates an empty PMGraphicalObject
@@ -68,6 +68,30 @@ public:
     * Sets the no_shadow flag
     */
    void setNoShadow( bool yes );
+   /**
+    * Returns the state of the no_image flag.
+    */
+   bool noImage( ) const { return m_noImage; }
+   /**
+    * Sets the no_image flag
+    */
+   void setNoImage( bool yes );
+   /**
+    * Returns the state of the no_reflection flag
+    */
+   bool noReflection( ) const { return m_noReflection; }
+   /**
+    * Sets the no_reflection flag
+    */
+   void setNoReflection( bool yes );
+   /**
+    * Returns the state of the double_illuminate flag
+    */
+   bool doubleIlluminate( ) const { return m_doubleIlluminate; }
+   /**
+    * Sets the double_illuminate flag
+    */
+   void setDoubleIlluminate( bool yes );
    /**
     * Returns the visibility level
     */
@@ -99,10 +123,14 @@ private:
    /**
     * IDs for @ref PMMementoData
     */
-   enum PMGraphicalObjectMementoID { PMNoShadowID, PMVisibilityID,
+   enum PMGraphicalObjectMementoID { PMNoShadowID, PMNoImageID, PMNoReflectionID,
+                                     PMDoubleIlluminateID, PMVisibilityID,
                                      PMRelativeVisibilityID, PMExportID };
 
    bool m_noShadow;
+   bool m_noImage;
+   bool m_noReflection;
+   bool m_doubleIlluminate;
    int m_visibilityLevel;
    bool m_relativeVisibility;
    bool m_export;
