@@ -40,7 +40,7 @@ PMHeightFieldEdit::PMHeightFieldEdit( QWidget* parent, const char* name )
 void PMHeightFieldEdit::createTopWidgets( )
 {
    Base::createTopWidgets( );
-   
+
    QHBoxLayout* hl = new QHBoxLayout( topLayout( ) );
    hl->addWidget( new QLabel( i18n( "Type:" ), this ) );
    m_pHeightFieldType = new QComboBox( false, this );
@@ -68,19 +68,13 @@ void PMHeightFieldEdit::createTopWidgets( )
    m_pWaterLevel->setValidation( true, 0.0, true, 1.0 );
    hl->addWidget( m_pWaterLevel );
    hl->addStretch( 1 );
-   
+
    m_pHierarchy = new QCheckBox( i18n( "Hierarchy" ), this );
    topLayout( )->addWidget( m_pHierarchy );
 
    m_pSmooth = new QCheckBox( i18n( "Smooth" ), this );
    topLayout( )->addWidget( m_pSmooth );
 
-   hl = new QHBoxLayout( topLayout( ) );
-   hl->addWidget( new QLabel( i18n( "Display detail:" ), this ) );
-   m_pDisplayDetail = new QSlider( 0, 256, 16, 0, Qt::Horizontal, this );
-   m_pDisplayDetail->setTracking( false );
-   hl->addWidget( m_pDisplayDetail );
-
    connect( m_pHeightFieldType, SIGNAL( activated( int ) ),
             SLOT( slotTypeChanged( int ) ) );
    connect( m_pFileName, SIGNAL( textChanged( const QString& ) ),
@@ -90,7 +84,6 @@ void PMHeightFieldEdit::createTopWidgets( )
    connect( m_pWaterLevel, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pHierarchy, SIGNAL( clicked( ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pSmooth, SIGNAL( clicked( ) ), SIGNAL( dataChanged( ) ) );
-   connect( m_pDisplayDetail, SIGNAL( valueChanged( int ) ), SIGNAL( dataChanged( ) ) );
 }
 
 void PMHeightFieldEdit::displayObject( PMObject* o )
@@ -128,15 +121,12 @@ void PMHeightFieldEdit::displayObject( PMObject* o )
       m_pWaterLevel->setValue( m_pDisplayedObject->waterLevel( ) );
       m_pHierarchy->setChecked( m_pDisplayedObject->hierarchy( ) );
       m_pSmooth->setChecked( m_pDisplayedObject->smooth( ) );
-      m_pDisplayDetail->setValue( m_pDisplayedObject->displayDetail( ) );
-
 
       m_pHeightFieldType->setEnabled( !readOnly );
       m_pFileName->setReadOnly( readOnly );
       m_pChooseFileName->setEnabled( !readOnly );
       m_pHierarchy->setEnabled( !readOnly );
       m_pSmooth->setEnabled( !readOnly );
-      m_pDisplayDetail->setEnabled( !readOnly );
 
       Base::displayObject( o );
    }
@@ -177,7 +167,6 @@ void PMHeightFieldEdit::saveContents( )
       m_pDisplayedObject->setWaterLevel( m_pWaterLevel->value( ) );
       m_pDisplayedObject->setHierarchy( m_pHierarchy->isChecked( ) );
       m_pDisplayedObject->setSmooth( m_pSmooth->isChecked( ) );
-      m_pDisplayedObject->setDisplayDetail( m_pDisplayDetail->value( ) );
    }
 }
 
