@@ -66,7 +66,7 @@ void PMLinkEdit::init( )
 
    QHBoxLayout* layout = new QHBoxLayout( );
    grid->addLayout( layout, 1, 1 );
-   m_pSelectButton = new QPushButton( i18n( "Select" ), this );
+   m_pSelectButton = new QPushButton( i18n( "Select..." ), this );
    layout->addWidget( m_pSelectButton );
    m_pClearButton = new KPushButton( KStdGuiItem::clear(), this );
    layout->addWidget( m_pClearButton );
