@@ -59,47 +59,43 @@ protected:
    virtual void saveContents( );
 
 protected slots:
-   void slotIntervalsClicked( );
-   void slotSamplesClicked( );
-   void slotConfidenceClicked( );
-   void slotVarianceClicked( );
-   void slotRatioClicked( );
+   void slotMethodChanged( int c );
    void slotAbsorptionClicked( );
    void slotEmissionClicked( );
    void slotScatteringClicked( );
-   void slotComboChanged( int c );
+   void slotScatteringTypeChanged( int c );
 
 private:
    PMMedia* m_pDisplayedObject;
+
+   QComboBox* m_pMethodEdit;
    PMIntEdit* m_pIntervalsEdit;
-   QLabel* m_pSamplesMinLabel;
    PMIntEdit* m_pSamplesMinEdit;
    QLabel* m_pSamplesMaxLabel;
    PMIntEdit* m_pSamplesMaxEdit;
    PMFloatEdit* m_pConfidenceEdit;
    PMFloatEdit* m_pVarianceEdit;
    PMFloatEdit* m_pRatioEdit;
-   
+
+   QWidget* m_pAAWidget;
+   PMIntEdit* m_pAALevelEdit;
+   PMFloatEdit* m_pAAThresholdEdit;
+
+   QCheckBox* m_pEnableAbsorptionEdit;
    PMColorEdit* m_pAbsorptionEdit;
    QLabel* m_pAbsorptionLabel;
+
+   QCheckBox* m_pEnableEmissionEdit;
    PMColorEdit* m_pEmissionEdit;
    QLabel* m_pEmissionLabel;
-   QLabel* m_pScatteringTypeLabel;
+
+   QCheckBox* m_pEnableScatteringEdit;
+   QWidget* m_pScatteringWidget;
    QComboBox* m_pScatteringTypeEdit;
-   QLabel* m_pScatteringColorLabel;
    PMColorEdit* m_pScatteringColorEdit;
    QLabel* m_pScatteringEccentricityLabel;
    PMFloatEdit* m_pScatteringEccentricityEdit;
-   QLabel* m_pScatteringExtinctionLabel;
    PMFloatEdit* m_pScatteringExtinctionEdit;
-   QCheckBox* m_pEnableIntervalsEdit;
-   QCheckBox* m_pEnableSamplesEdit;
-   QCheckBox* m_pEnableConfidenceEdit;
-   QCheckBox* m_pEnableVarianceEdit;
-   QCheckBox* m_pEnableRatioEdit;
-   QCheckBox* m_pEnableAbsorptionEdit;
-   QCheckBox* m_pEnableEmissionEdit;
-   QCheckBox* m_pEnableScatteringEdit;
 };
 
 
