@@ -95,16 +95,25 @@ public:
     * Sets the normal's accuracy
     */
    void setAccuracy( double c );
+   /**
+    * Returns the uv mapping flag
+    */
+   bool uvMapping( ) const { return m_uvMapping; }
+   /**
+    * Sets the uv mapping flag
+    */
+   void setUVMapping( bool m );
 
 private:
    /**
     * IDs for @ref PMMementoData
     */
-   enum PMNormalMementoID { PMEnableBumpSizeID, PMBumpSizeID, PMAccuracyID };
+   enum PMNormalMementoID { PMEnableBumpSizeID, PMBumpSizeID, PMAccuracyID, PMUVMappingID };
 
    bool m_enableBumpSize;
    double m_bumpSize;
    double m_accuracy;
+   bool m_uvMapping;
 
    static PMMetaObject* s_pMetaObject;
 };
