@@ -53,9 +53,13 @@ void PMNormalEdit::createTopWidgets( )
    hl->addWidget( m_pAccuracy );
    hl->addStretch( 1 );
 
+   m_pUVMapping = new QCheckBox( i18n( "UV mapping" ), this );
+   topLayout( )->addWidget( m_pUVMapping );
+
    connect( m_pBumpSizeCheck, SIGNAL( clicked( ) ), SLOT( slotBumpSizeClicked( )) );
    connect( m_pBumpSizeEdit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( )) );
    connect( m_pAccuracy, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
+   connect( m_pUVMapping, SIGNAL( clicked( ) ), SIGNAL( dataChanged( ) ) );
 }
 
 void PMNormalEdit::displayObject( PMObject* o )
@@ -72,6 +76,8 @@ void PMNormalEdit::displayObject( PMObject* o )
       slotBumpSizeClicked( );
       m_pAccuracy->setValue( m_pDisplayedObject->accuracy( ) );
       m_pAccuracy->setReadOnly( readOnly );
+      m_pUVMapping->setChecked( m_pDisplayedObject->uvMapping() );
+      m_pUVMapping->setEnabled( !readOnly );
 
       Base::displayObject( o );
    }
@@ -87,6 +93,7 @@ void PMNormalEdit::saveContents( )
       m_pDisplayedObject->enableBumpSize( m_pBumpSizeCheck->isChecked( ) );
       m_pDisplayedObject->setBumpSize( m_pBumpSizeEdit->value( ) );
       m_pDisplayedObject->setAccuracy( m_pAccuracy->value( ) );
+      m_pDisplayedObject->setUVMapping( m_pUVMapping->isChecked() );
    }
 }
 
