@@ -18,6 +18,7 @@
 #include "pmpovrayrenderwidget.h"
 #include "pmdefaults.h"
 #include "pmdebug.h"
+#include "pmdragwidget.h"
 
 #include <kprocess.h>
 #include <kmessagebox.h>
@@ -28,6 +29,7 @@
 #include <qcolor.h>
 #include <qpainter.h>
 #include <qtextstream.h>
+#include <qdragobject.h>
 
 #ifdef KDE_NO_COMPAT
 #undef KDE_NO_COMPAT
@@ -39,7 +41,7 @@ QString PMPovrayRenderWidget::s_povrayCommand = c_defaultPovrayCommand;
 QStringList PMPovrayRenderWidget::s_libraryPaths;
 
 PMPovrayRenderWidget::PMPovrayRenderWidget( QWidget* parent, const char* name )
-      : QWidget( parent, name )
+      : PMDragWidget( parent, name )
 {
    m_pProcess = 0;
    m_bSuspended = false;
@@ -426,4 +428,10 @@ void PMPovrayRenderWidget::restoreConfig( KConfig* cfg )
 #endif
 }
 
+void PMPovrayRenderWidget::startDrag( )
+{
+   QImageDrag* d = new QImageDrag( m_image, this );
+   d->dragCopy( );
+}
+
 #include "pmpovrayrenderwidget.moc"
