@@ -96,6 +96,8 @@ public:
    /** */
    virtual bool multipleSelectControlPoints( ) const { return true; }
    /** */
+   virtual bool hasDisplayDetail( ) const { return true; }
+   /** */
    virtual void addObjectActions( const PMControlPointList&,
                                   QPtrList<PMObjectAction>& );
    /** */
@@ -178,7 +180,7 @@ protected:
    /** */
    virtual void createViewStructure( );
    /** */
-   virtual int viewStructureParameterKey( ) const { return s_parameterKey; }
+   virtual int viewStructureParameterKey( ) const { return s_parameterKey + globalDetailKey( ); }
 
 private:
    /**
