@@ -88,7 +88,7 @@ void PMPrismEdit::createBottomWidgets( )
 {
    m_pEditWidget = new QWidget( this );
    topLayout( )->addWidget( m_pEditWidget );
-   m_pOpen = new QCheckBox( i18n( "Open" ), this );
+   m_pOpen = new QCheckBox( i18n( "type of the object", "Open" ), this );
    topLayout( )->addWidget( m_pOpen );
    m_pSturm = new QCheckBox( i18n( "Sturm" ), this );
    topLayout( )->addWidget( m_pSturm );
