@@ -58,7 +58,7 @@ PMRenderModesDialog::PMRenderModesDialog( PMRenderModeList* modes, QWidget* pare
    connect( m_pAddButton, SIGNAL( clicked( ) ), SLOT( slotAdd( ) ) );
    m_pRemoveButton = new QPushButton( i18n( "Remove" ), buttons );
    connect( m_pRemoveButton, SIGNAL( clicked( ) ), SLOT( slotRemove( ) ) );
-   m_pEditButton = new QPushButton( i18n( "Edit" ), buttons );
+   m_pEditButton = new QPushButton( i18n( "Edit..." ), buttons );
    connect( m_pEditButton, SIGNAL( clicked( ) ), SLOT( slotEdit( ) ) );
    m_pUpButton = new QPushButton( i18n( "Up" ), buttons );
    connect( m_pUpButton, SIGNAL( clicked( ) ), SLOT( slotUp( ) ) );
@@ -447,7 +447,7 @@ bool PMRenderModeDialog::validate( )
    if( m_pDescriptionEdit->text( ).isEmpty( ) )
    {
       KMessageBox::error( this, i18n( "Please enter a description for the "
-                                      "render mode" ), i18n( "Error" ) );
+                                      "render mode." ), i18n( "Error" ) );
       m_pDescriptionEdit->selectAll( );
       return false;
    }
