@@ -26,7 +26,7 @@
 const double heightDefault = 0;
 const double slopeDefault = 0;
 
-PMDefinePropertyClass( PMSlope, PMProperty );
+PMDefinePropertyClass( PMSlope, PMSlopeProperty );
 
 PMMetaObject* PMSlope::s_pMetaObject = 0;
 PMObject* createNewSlope( PMPart* part )
@@ -76,9 +76,9 @@ PMMetaObject* PMSlope::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Slope", Base::metaObject( ),
                                         createNewSlope );
       s_pMetaObject->addProperty(
-         new PMProperty( "height", &PMSlope::setHeight, &PMSlope::height ) );
+         new PMSlopeProperty( "height", &PMSlope::setHeight, &PMSlope::height ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "slope", &PMSlope::setSlope, &PMSlope::slope ) );
+         new PMSlopeProperty( "slope", &PMSlope::setSlope, &PMSlope::slope ) );
    }
    return s_pMetaObject;
 }
