@@ -51,7 +51,7 @@ PMObject* createNewSurfaceOfRevolution( PMPart* part )
    return new PMSurfaceOfRevolution( part );
 }
 
-PMDefinePropertyClass( PMSurfaceOfRevolution, PMProperty );
+PMDefinePropertyClass( PMSurfaceOfRevolution, PMSurfaceOfRevolutionProperty );
 
 class PMPointProperty : public PMPropertyBase
 {
@@ -80,10 +80,10 @@ protected:
       PMVector v = var.vectorData( );
       v.resize( 2 );
 
-      for( i = 0; i < m_index && it != list.end( ); i++ )
+      for( i = 0; i < m_index && it != list.end( ); ++i )
          ++it;
       // expand the list if necessary
-      for( ; i < m_index; i++ )
+      for( ; i < m_index; ++i )
          list.insert( it, v );
       if( it == list.end( ) )
          it = list.insert( it, v );
@@ -117,7 +117,7 @@ PMSurfaceOfRevolution::PMSurfaceOfRevolution( PMPart* part )
 {
    int i;
 
-   for( i = 0; i < defaultNumberOfPoints; i++ )
+   for( i = 0; i < defaultNumberOfPoints; ++i )
       m_points.append( defaultPoint[i] );
    m_sturm = defaultSturm;
    m_open = defaultOpen;
@@ -201,10 +201,10 @@ PMMetaObject* PMSurfaceOfRevolution::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "SurfaceOfRevolution", Base::metaObject( ),
                                         createNewSurfaceOfRevolution );
       s_pMetaObject->addProperty(
-         new PMProperty( "sturm", &PMSurfaceOfRevolution::setSturm,
+         new PMSurfaceOfRevolutionProperty( "sturm", &PMSurfaceOfRevolution::setSturm,
                          &PMSurfaceOfRevolution::sturm ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "open", &PMSurfaceOfRevolution::setOpen,
+         new PMSurfaceOfRevolutionProperty( "open", &PMSurfaceOfRevolution::setOpen,
                          &PMSurfaceOfRevolution::open ) );
       s_pMetaObject->addProperty( new PMPointProperty( ) );
    }
@@ -304,6 +304,9 @@ void PMSurfaceOfRevolution::createViewStructure( )
    if( s_rSteps == 0 )
       s_rSteps = c_defaultSurfaceOfRevolutionRSteps;
 
+   int rSteps = (int)( ( (float)s_rSteps / 2 ) * ( displayDetail( ) + 1 ) );
+   int sSteps = (int)( ( (float)s_sSteps / 2 ) * ( displayDetail( ) + 1 ) );
+
    int np = m_points.count( );
    int i, j, r;
 
@@ -311,9 +314,9 @@ void PMSurfaceOfRevolution::createViewStructure( )
    int ns = np - 3;
 
    // calculate number of points and lines of the view structure
-   int vsp = ns * s_sSteps + 1;
-   int vsl = ( 2 * vsp - 1 ) * s_rSteps;
-   vsp *= s_rSteps;
+   int vsp = ns * sSteps + 1;
+   int vsl = ( 2 * vsp - 1 ) * rSteps;
+   vsp *= rSteps;
 
    if( m_pViewStructure )
    {
@@ -334,28 +337,28 @@ void PMSurfaceOfRevolution::createViewStructure( )
    it3 = it2; ++it3;
    it4 = it3; ++it4;
 
-   for( i = 0; i < ns; i++, ++it1, ++it2, ++it3, ++it4 )
+   for( i = 0; i < ns; ++i, ++it1, ++it2, ++it3, ++it4 )
       segments.append( PMSorSegment( *it1, *it2, *it3, *it4 ) );
 
    // create the line array
    PMLineArray& lines = m_pViewStructure->lines( );
-   int vl = ns * s_sSteps;
+   int vl = ns * sSteps;
    int lb = 0;
-   for( i = 0; i < vl + 1; i++ )
+   for( i = 0; i < vl + 1; ++i )
    {
-      for( j = 0; j < s_rSteps - 1; j++ )
+      for( j = 0; j < rSteps - 1; ++j )
          lines[lb+j] = PMLine( lb + j, lb + j + 1 );
-      lines[lb+s_rSteps-1] = PMLine( lb, lb + s_rSteps - 1 );
-      lb += s_rSteps;
+      lines[lb+rSteps-1] = PMLine( lb, lb + rSteps - 1 );
+      lb += rSteps;
    }
    int pi = 0;
-   for( i = 0; i < vl; i++ )
+   for( i = 0; i < vl; ++i )
    {
-      for( j = 0; j < s_rSteps; j++ )
+      for( j = 0; j < rSteps; ++j )
       {
-         lines[lb] = PMLine( pi, pi + s_rSteps );
-         pi++;
-         lb++;
+         lines[lb] = PMLine( pi, pi + rSteps );
+         ++pi;
+         ++lb;
       }
    }
 
@@ -363,26 +366,26 @@ void PMSurfaceOfRevolution::createViewStructure( )
    PMVector point2, point3;
    QValueList<PMSorSegment>::Iterator sit = segments.begin( );
 
-   double poffset = 1.0 / s_sSteps;
-   PMMatrix rot = PMMatrix::rotation( 0.0, M_PI * 2.0 / s_rSteps, 0.0 );
+   double poffset = 1.0 / sSteps;
+   PMMatrix rot = PMMatrix::rotation( 0.0, M_PI * 2.0 / rSteps, 0.0 );
    PMPointArray& points = m_pViewStructure->points( );
    pi = 0;
 
    for( i = 0; i < ns; ++i, ++sit )
    {
-      for( j = 0; j < s_sSteps; j++ )
+      for( j = 0; j < sSteps; ++j )
       {
          point2 = ( *sit ).point( poffset * j );
          point3[0] = point2[0];
          point3[1] = point2[1];
          point3[2] = 0.0;
 
-         for( r = 0; r < s_rSteps; r++ )
+         for( r = 0; r < rSteps; ++r )
          {
             points[pi] = PMPoint( point3 );
-            if( r != s_rSteps - 1 )
+            if( r != rSteps - 1 )
                point3.transform( rot );
-            pi++;
+            ++pi;
          }
       }
       if( i == ns - 1 )
@@ -392,12 +395,12 @@ void PMSurfaceOfRevolution::createViewStructure( )
          point3[1] = point2[1];
          point3[2] = 0.0;
 
-         for( r = 0; r < s_rSteps; r++ )
+         for( r = 0; r < rSteps; ++r )
          {
             points[pi] = PMPoint( point3 );
-            if( r != s_rSteps - 1 )
+            if( r != rSteps - 1 )
                point3.transform( rot );
-            pi++;
+            ++pi;
          }
       }
    }
@@ -414,7 +417,7 @@ void PMSurfaceOfRevolution::controlPoints( PMControlPointList& list )
    PMSorControlPoint* lastPoint = 0;
    cp = 0;
 
-   for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, i++ )
+   for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, ++i )
    {
       lastPoint = cp;
       cp = new PMSorControlPoint( lastPoint, *it, PMSorControlPoint::PM2DXY, i,
@@ -425,7 +428,7 @@ void PMSurfaceOfRevolution::controlPoints( PMControlPointList& list )
    lastPoint = 0;
    cp = 0;
 
-   for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, i++ )
+   for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, ++i )
    {
       lastPoint = cp;
       cp = new PMSorControlPoint( lastPoint, *it, PMSorControlPoint::PM2DZY, i,
@@ -578,11 +581,11 @@ void PMSurfaceOfRevolution::splitSegment( const PMControlPointList& /*cp*/,
    QPtrListIterator<PMVector> it2( cpViewPosition );
    ++it2;
 
-   for( j = 0; j < 2; j++ )
+   for( j = 0; j < 2; ++j )
    {
       ++it1;
       ++it2;
-      for( i = 1; i < ( nump - 1 ); i++ )
+      for( i = 1; i < ( nump - 1 ); ++i )
       {
          mid = ( **it1 + **it2 ) / 2.0;
          dist[0] = mid[0];
@@ -612,7 +615,7 @@ void PMSurfaceOfRevolution::splitSegment( const PMControlPointList& /*cp*/,
 
    // calculate the spline segment
    --hit;
-   for( i = 0; i < 4; i++, ++hit )
+   for( i = 0; i < 4; ++i, ++hit )
       p[i] = *hit;
    PMSorSegment segment( p[0], p[1], p[2], p[3] );
 
@@ -652,9 +655,9 @@ void PMSurfaceOfRevolution::joinSegments( const PMControlPointList& /*cp*/,
 
    QPtrListIterator<PMVector> it1( cpViewPosition );
 
-   for( j = 0; j < 2; j++ )
+   for( j = 0; j < 2; ++j )
    {
-      for( i = 0; i < nump; i++ )
+      for( i = 0; i < nump; ++i )
       {
          p = *it1;
          dist[0] = (*p)[0];
@@ -677,9 +680,9 @@ void PMSurfaceOfRevolution::joinSegments( const PMControlPointList& /*cp*/,
 
    // never remove the first or last point
    if( ns == 0 )
-      ns++;
+      ++ns;
    if( ns == ( nump - 1 ) )
-      ns--;
+      --ns;
    it = newPoints.at( ns );
    newPoints.remove( it );
 
@@ -692,7 +695,7 @@ void PMSurfaceOfRevolution::setRSteps( int r )
       s_rSteps = r;
    else
       kdDebug( PMArea ) << "PMSurfaceOfRevolution::setRSteps: R must be greater than 3\n";
-   s_parameterKey++;
+   ++s_parameterKey;
 }
 
 void PMSurfaceOfRevolution::setSSteps( int s )
@@ -701,5 +704,5 @@ void PMSurfaceOfRevolution::setSSteps( int s )
       s_sSteps = s;
    else
       kdDebug( PMArea ) << "PMSurfaceOfRevolution::setSSteps: S must be greater than 0\n";
-   s_parameterKey++;
+   ++s_parameterKey;
 }
