@@ -21,7 +21,7 @@
 #include "pmlinkedit.h"
 
 #include <qlayout.h>
-#include <qlabel.h>
+#include <qcheckbox.h>
 #include <klocale.h>
 
 
@@ -31,14 +31,37 @@ PMTextureEdit::PMTextureEdit( QWidget* parent, const char* name )
    m_pDisplayedObject = 0;
 }
 
+void PMTextureEdit::createTopWidgets()
+{
+   Base::createTopWidgets();
+   m_pUVMapping = new QCheckBox( i18n( "UV mapping" ), this );
+   topLayout( )->addWidget( m_pUVMapping );
+
+   connect( m_pUVMapping, SIGNAL( clicked() ), SIGNAL( dataChanged() ) );
+}
+
 void PMTextureEdit::displayObject( PMObject* o )
 {
    if( o->isA( "Texture" ) )
    {
+      m_pDisplayedObject = ( PMTexture* ) o;
+      bool readOnly = m_pDisplayedObject->isReadOnly( );
+      m_pUVMapping->setChecked( m_pDisplayedObject->uvMapping() );
+      m_pUVMapping->setEnabled( !readOnly );
+
       Base::displayObject( o );
    }
    else
       kdError( PMArea ) << "PMTextureEdit: Can't display object\n";
 }
 
+void PMTextureEdit::saveContents()
+{
+   if( m_pDisplayedObject )
+   {
+      Base::saveContents();
+      m_pDisplayedObject->setUVMapping( m_pUVMapping->isChecked() );
+   }
+}
+
 #include "pmtextureedit.moc"
