@@ -27,6 +27,7 @@
 #include "pmtexturebaseedit.h"
 
 class PMTexture;
+class QCheckBox;
 
 /**
  * Dialog edit class for @ref PMTexture
@@ -46,12 +47,13 @@ public:
 
 protected:
    /** */
-//   virtual void createTopWidgets( );
+   virtual void createTopWidgets( );
    /** */
-//   virtual void saveContents( );
+   virtual void saveContents( );
 
 private:
    PMTexture* m_pDisplayedObject;
+   QCheckBox* m_pUVMapping;
 };
 
 
