@@ -180,19 +180,16 @@ void PMTreeView::slotObjectChanged( PMObject* obj, const int mode,
       {
          clearSelection( );
 
-         if( obj )
-         {
-            if( !pTreeItem )
-               pTreeItem = findObject( obj );
+         if( !pTreeItem )
+             pTreeItem = findObject( obj );
 
-            if( pTreeItem )
-            {
-               PMTreeViewItem* p;
-               for( p = pTreeItem->parent( ); p; p = p->parent( ) )
-                  p->setOpen( true );
-               pTreeItem->setSelected( true );
-               setCurrentItem( pTreeItem );
-            }
+         if( pTreeItem )
+         {
+             PMTreeViewItem* p;
+             for( p = pTreeItem->parent( ); p; p = p->parent( ) )
+                 p->setOpen( true );
+             pTreeItem->setSelected( true );
+             setCurrentItem( pTreeItem );
          }
       }
       if( mode & PMCDeselected )
