@@ -98,6 +98,23 @@ public:
     */
    void setNormal( int i, const PMVector& n );
 
+   /**
+    * Returns the uv vector with index i
+    */
+   PMVector uvVector( int i ) const;
+   /**
+    * Sets the uv vector with index i
+    */
+   void setUVVector( int i, const PMVector& v );
+   /**
+    * Return true if triangle has uv vectors
+    */
+   bool isUVEnabled( ) const { return m_uvEnabled; }
+   /**
+    * Enables/disable the uv vectors
+    */
+   void enableUV( bool yes );
+
    /** */
    virtual void restoreMemento( PMMemento* s );
    /** */
@@ -121,10 +138,14 @@ protected:
     */
    enum PMTriangleMementoID { PMPoint0ID, PMPoint1ID, PMPoint2ID,
                               PMNormal0ID, PMNormal1ID, PMNormal2ID,
-                              PMSmoothID };
+                              PMSmoothID,
+                              PMUVVector0ID, PMUVVector1ID, PMUVVector2ID,
+                              PMUVEnabledID };
    PMVector m_point[3];
    PMVector m_normal[3];
+   PMVector m_uvVector[3];
    bool m_smooth;
+   bool m_uvEnabled;
 
    /**
     * The default view structure. It can be shared between triangles
