@@ -290,7 +290,7 @@ void PMViewLayout::saveData( QDomElement& e, QDomDocument& doc ) const
    }
 }
 
-void PMViewLayout::setName( QString n )
+void PMViewLayout::setName( const QString& n )
 {
    m_name = n;
 }
@@ -329,7 +329,7 @@ void PMViewLayout::displayLayout( PMShell* shell )
                            QPoint( ( *it ).floatingPositionX( ),
                                    ( *it ).floatingPositionY( ) ) );
          dock->resize( ( *it ).floatingWidth( ), ( *it ).floatingHeight( ) );
-			dock->show( );
+         dock->show( );
       }
       else
       {
@@ -610,7 +610,7 @@ void PMViewLayout::recursiveExtractOneColumn(
          else
          {
             // a kpovmodeler view???
-				if( w->inherits( "PMViewBase" ) )
+            if( w->inherits( "PMViewBase" ) )
             {
                PMViewBase* v = ( PMViewBase* ) w;
                ( *eit ).setViewType( v->viewType( ) );
