@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <math.h>
@@ -175,6 +175,7 @@ double SVGAnimationElementImpl::parseClockValue(const QString &data) const
 }
 
 /*
+@namespace KSVG
 @begin SVGAnimationElementImpl::s_hashTable 23 
  targetElement	SVGAnimationElementImpl::TargetElement	DontDelete|ReadOnly
  href			SVGAnimationElementImpl::Href			DontDelete|ReadOnly
@@ -199,6 +200,7 @@ double SVGAnimationElementImpl::parseClockValue(const QString &data) const
  repeatDur		SVGAnimationElementImpl::RepeatDur		DontDelete|ReadOnly
  fill			SVGAnimationElementImpl::Fill			DontDelete|ReadOnly
 @end
+@namespace KSVG
 @begin SVGAnimationElementImplProto::s_hashTable 5
  getStartTime		SVGAnimationElementImpl::GetStartTime		DontDelete|Function 0
  getCurrentTime		SVGAnimationElementImpl::GetCurrentTime		DontDelete|Function 0
