@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <qxml.h>
@@ -184,6 +184,7 @@ void SVGElementImpl::defaultEventHandler(SVGEventImpl *)
 }
 
 /*
+@namespace KSVG
 @begin SVGElementImpl::s_hashTable 23
  id						SVGElementImpl::ElementId 		DontDelete
  ownerSVGElement		SVGElementImpl::OwnerSvgElement	DontDelete|ReadOnly
@@ -207,6 +208,7 @@ void SVGElementImpl::defaultEventHandler(SVGEventImpl *)
  onerror				SVGElementImpl::OnError			DontDelete
  onabort				SVGElementImpl::OnAbort			DontDelete
 @end
+@namespace KSVG
 @begin SVGElementImplProto::s_hashTable 5
  getStyle				SVGElementImpl::GetStyle			DontDelete|Function 0
  setProperty			SVGElementImpl::SetProperty			DontDelete|Function 2
