@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef SVGHelperImpl_H
@@ -25,6 +25,8 @@
 
 #include "SVGElementImpl.h"
 #include "SVGLengthImpl.h"
+#include "SVGDocumentImpl.h"
+#include "ksvg_ecma.h"
 
 #include "ksvg_lookup.h"
 
@@ -79,7 +81,7 @@ public:
 	static QRect fromUserspace(SVGElementImpl *, const QRect &);
 };
 
-};
+}
 
 #endif
 
