@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <kdebug.h>
@@ -691,6 +691,7 @@ float SVGStylableImpl::fontSizeForText(const QString &value)
 // Ecma stuff
 
 /*
+@namespace KSVG
 @begin SVGStylableImpl::s_hashTable 47
  className			SVGStylableImpl::ClassName			DontDelete|ReadOnly
  style				SVGStylableImpl::Style				DontDelete|ReadOnly
@@ -738,6 +739,7 @@ float SVGStylableImpl::fontSizeForText(const QString &value)
  color-interpolation	SVGStylableImpl::ColorInterpolation	DontDelete|ReadOnly
  mask				SVGStylableImpl::Mask	DontDelete|ReadOnly
 @end
+@namespace KSVG
 @begin SVGStylableImplProto::s_hashTable 2
  getStyle			SVGStylableImpl::GetStyle			DontDelete|Function 0
 @end
