@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     aint with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef T2P_FONT_H
@@ -28,13 +28,11 @@
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
-#include <kdemacros.h>
-
 namespace T2P
 {
 	class Converter;
 
-	class KDE_EXPORT FontVisualParams
+	class FontVisualParams
 	{
 	public:
 		FontVisualParams();
@@ -61,7 +59,7 @@ namespace T2P
 		std::list<std::string> m_fontList;
 	};
 
-	class KDE_EXPORT Font
+	class Font
 	{
 	public:
 		Font(Converter *context);
@@ -86,7 +84,7 @@ namespace T2P
 
 		bool m_ready;
 	};
-};
+}
 
 #endif
 
