@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     aint with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef T2P_GLYPH_H
@@ -32,14 +32,12 @@
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
-#include <kdemacros.h>
-
 namespace T2P
 {
 	class Font;
 	
 	// Represent one single glyph in cache
-	class KDE_EXPORT Glyph
+	class Glyph
 	{
 	public:
 		Glyph();
@@ -62,7 +60,7 @@ namespace T2P
 
 	// Is created for every character which needs to
 	// be rendered and has to be deleted by the "client"
-	class KDE_EXPORT GlyphAffinePair
+	class GlyphAffinePair
 	{
 	public:
 		GlyphAffinePair(const Glyph *glyph, const Affine &affine);
@@ -80,7 +78,7 @@ namespace T2P
 		Affine m_affine;
 	};
 
-	class KDE_EXPORT GlyphSet
+	class GlyphSet
 	{
 	public:
 		GlyphSet();
@@ -124,7 +122,7 @@ namespace T2P
 		std::list<float> m_glyphYAdvance;
 	};
 
-	class KDE_EXPORT GlyphLayoutParams
+	class GlyphLayoutParams
 	{
 	public:
 		GlyphLayoutParams();
@@ -164,7 +162,7 @@ namespace T2P
 		std::string m_baseline; // baseline description, using same system as svg
 	};
 
-	class KDE_EXPORT GlyphRenderParams
+	class GlyphRenderParams
 	{
 	public:
 		GlyphRenderParams();
@@ -193,7 +191,7 @@ namespace T2P
 		unsigned int m_lastGlyph;	// Kerning
 		unsigned short m_character; // Unicode glyph to process
 	};
-};
+}
 
 #endif
 
