@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <krun.h>
@@ -50,7 +50,7 @@ using namespace KSVG;
 #define DEFAULT_HEIGHT 400
 #define ZOOM_FACTOR 1.2
 
-struct KSVGPlugin::Private 
+struct KSVGPlugin::Private
 {
 	KSVGWidget *window;
 	KSVGPluginBrowserExtension *extension;
@@ -118,7 +118,7 @@ KSVGPlugin::KSVGPlugin(QWidget *wparent, const char *, QObject *parent, const ch
 	ksvgd->stopAnimationsAction = new KAction(i18n("&Stop Animations"), "stop", Key_Escape, this, SLOT(slotStop()), actionCollection(), "stop_anims");
 	ksvgd->viewSourceAction = new KAction(i18n("View &Source"), "document2", Key_F6, this, SLOT(slotViewSource()), actionCollection(), "view_source");
 	ksvgd->viewMemoryAction = new KAction(i18n("View &Memory"), "document2", Key_F7, this, SLOT(slotViewMemory()), actionCollection(), "view_memory");
-	ksvgd->saveToPNG = new KAction(i18n("Save to PNG"), "save", 0, this, SLOT(slotSaveToPNG()), actionCollection(), "save_to_png");
+	ksvgd->saveToPNG = new KAction(i18n("Save to PNG..."), "save", 0, this, SLOT(slotSaveToPNG()), actionCollection(), "save_to_png");
 //	ksvgd->aboutApp = KStdAction::aboutApp(this, SLOT(slotAboutKSVG()), actionCollection());//, "KSVG");
 	ksvgd->aboutApp = new KAction(i18n("About KSVG"), "vectorgfx", 0, this, SLOT(slotAboutKSVG()), actionCollection(), "help_about_app");
 	ksvgd->fontKerningAction = new KToggleAction(i18n("Use Font &Kerning"), "viewmagfit", Key_F8, this, SLOT(slotFontKerning()), actionCollection(), "font_kerning");
@@ -136,11 +136,11 @@ KSVGPlugin::KSVGPlugin(QWidget *wparent, const char *, QObject *parent, const ch
 	QPtrListIterator<KSVG::CanvasInfo> it(canvasList);
 	KSVG::CanvasInfo *canvasInfo = 0;
 	while((canvasInfo = it.current()) != 0)
-	{	
+	{
 		items << canvasInfo->name;
 		++it;
 	}
-	
+
 	ksvgd->renderingBackendAction->setItems(items);
 	ksvgd->renderingBackendAction->setCurrentItem(KSVG::CanvasFactory::self()->itemInList(ksvgd->canvas));
 
@@ -175,7 +175,7 @@ SVGDocumentImpl *KSVGPlugin::docImpl()
 SVGDocument KSVGPlugin::document()
 {
 	return SVGDocument(ksvgd->doc);
-}		
+}
 
 void KSVGPlugin::reset()
 {
@@ -196,7 +196,7 @@ bool KSVGPlugin::openURL(const KURL &url)
 		reset();
 
 		DocumentFactory *docFactory = DocumentFactory::self();
-		
+
 		// when embedded, stick to passed width and height
 		ksvgd->doc = docFactory->requestDocumentImpl((ksvgd->width > 0 && ksvgd->height > 0));
 
