@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef IMBLIBCONFIG_H
@@ -37,11 +37,13 @@ public:
 
   // -----------------------
 
-  bool 		ownPalette;
-  bool 		fastRemap;
-  bool 		fastRender;
-  bool		dither16bit;
-  bool 		dither8bit;
+  bool 		ownPalette		:1;
+  bool 		fastRemap		:1;
+  bool 		fastRender		:1;
+  bool		dither16bit		:1;
+  bool 		dither8bit		:1;
+  bool		smoothScale		:1;
+  
 
   uint          gammaFactor;
   uint          brightnessFactor;
