@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qcheckbox.h>
@@ -40,7 +40,8 @@ SlideShowWidget::SlideShowWidget( QWidget *parent, const char *name )
     m_delayTime = new KIntNumInput( this, "delay time" );
     m_delayTime->setLabel( i18n("De&lay between slides:") );
     m_delayTime->setSuffix( i18n(" sec") );
-    m_delayTime->setRange( 1, 60 * 60 ); // 1 hour
+    m_delayTime->setRange( 0, 60 * 60 ); // 1 hour
+    m_delayTime->setSpecialValueText( i18n("Wait for key") );
 
     m_cycles = new KIntNumInput( m_delayTime, 1, this );
     m_cycles->setLabel( i18n("&Iterations (0 = infinite):") );
