@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -807,15 +807,15 @@ void KViewViewer::slotFileDirty( const QString & )
 				QString::null, kapp->miniIcon() );
 		( void )new QLabel( i18n( "The image %1 which you have modified has changed on disk.\n"
 								  "Do you want to reload the file and lose your changes?\n"
-								  "If you choose No and subsequently save the image, you will lose the\n"
+								  "If you don't and subsequently save the image, you will lose the\n"
 								  "changes that have already been saved." ).arg( url().fileName() ), vb );
 		QWidget * hb = new QWidget( vb );
 		QHBoxLayout * layout = new QHBoxLayout( hb );
 		layout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum ) );
-		KPushButton * yes = new KPushButton( KStdGuiItem::yes(), hb );
+		KPushButton * yes = new KPushButton( i18n("Reload"), hb );
 		layout->addWidget( yes );
 		layout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum ) );
-		KPushButton * no = new KPushButton( KStdGuiItem::no(), hb );
+		KPushButton * no = new KPushButton( i18n("Do Not Reload"), hb );
 		layout->addWidget( no );
 		layout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum ) );
 		connect( yes, SIGNAL( clicked() ), this, SLOT( slotReloadUnmodified() ) );
