@@ -9,7 +9,7 @@
 void ImageListDialog::init()
 {
     kdDebug( 4630 ) << k_funcinfo << endl;
-    m_pInterval->setRange( 0, 60000 );
+    m_pInterval->setRange( 0, 60000, 1000 );
     noSort();
 }
 
