@@ -1,3 +1,4 @@
+// -*- C++ -*-
 // ***************************************************************************
 //
 // Copyright (C) 2005 by Wilfried Huss <Wilfried.Huss@gmx.at>
@@ -14,26 +15,27 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software 
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // ***************************************************************************
 
 #ifndef _EMPTYRENDERER_H_
 #define _EMPTYRENDERER_H_
 
-#include "documentPage.h"
 #include "documentRenderer.h"
-#include "pageSize.h"
 
-class EmptyRenderer : public documentRenderer
+class RenderedDocumentPage;
+
+
+class EmptyRenderer : public DocumentRenderer
 {
   Q_OBJECT
 
 public:
   EmptyRenderer(QWidget* parent);
 
-  virtual bool setFile(const QString& fname) { Q_UNUSED(fname); return false; };
+  virtual bool setFile(const QString &, const KURL &) { return false; }
 
-  void drawPage(double res, documentPage* page) { Q_UNUSED(res); Q_UNUSED(page); };
+  void drawPage(double res, RenderedDocumentPage* page) { Q_UNUSED(res); Q_UNUSED(page); }
 };
 
 #endif
