@@ -7,6 +7,8 @@
 
 // Add header files alphabetically
 
+#include <config.h>
+
 #include <kcombobox.h>
 #include <kdebug.h>
 #include <klocale.h>
@@ -24,7 +26,7 @@
 pageSizeWidget::pageSizeWidget( QWidget* parent,  const char* name, WFlags fl )
     : pageSizeWidget_base( parent,  name, fl )
 {
-  connect(&chosenSize, SIGNAL(sizeChanged(float, float)), previewer, SLOT(setSize(float, float)));
+  connect(&chosenSize, SIGNAL(sizeChanged(const SimplePageSize&)), previewer, SLOT(setSize(const SimplePageSize&)));
 
   // Set up the formatChoice QComboBox
   formatChoice->insertItem(i18n("Custom Size"));
@@ -52,11 +54,11 @@ pageSizeWidget::pageSizeWidget( QWidget* parent,  const char* name, WFlags fl )
   connect(widthInput, SIGNAL(textChanged(const QString &)), this, SLOT(input(const QString &)));
   connect(heightInput, SIGNAL(textChanged(const QString &)), this, SLOT(input(const QString &)));
 
-  // Allow entries between 0 and 500. More filtering is done by the
+  // Allow entries between 0 and 1200. More filtering is done by the
   // pageSize class, which silently ignores values which are out of
   // range.
-  widthInput->setValidator(new QDoubleValidator(0.0, 500.0, 1, this, "widthValidator"));
-  heightInput->setValidator(new QDoubleValidator(0.0, 500.0, 1, this, "heightValidator"));
+  widthInput->setValidator(new QDoubleValidator(0.0, 1200.0, 1, this, "widthValidator"));
+  heightInput->setValidator(new QDoubleValidator(0.0, 1200.0, 1, this, "heightValidator"));
 }
 
 
@@ -79,7 +81,7 @@ void pageSizeWidget::paperSize(int index)
 }
 
 
-void pageSizeWidget::setPageSize(QString sizeName)
+void pageSizeWidget::setPageSize(const QString& sizeName)
 {
   chosenSize.setPageSize(sizeName);
 
@@ -97,7 +99,7 @@ void pageSizeWidget::setPageSize(QString sizeName)
 }
 
 
-void pageSizeWidget::fillTextFields(void)
+void pageSizeWidget::fillTextFields()
 {				   
   // Warning! It is really necessary here to first generate both
   // strings, and then call setText() on the input widgets. Reason?
