@@ -1,4 +1,5 @@
-// sizePreview.h
+// -*- C++ -*-
+// SizePreview.h
 //
 // Part of KVIEWSHELL - A framework for multipage text/gfx viewers
 //
@@ -10,26 +11,28 @@
 #ifndef SIZEPREVIEW_H
 #define SIZEPREVIEW_H
 
-class QPaintEvent;
-
+#include <qpixmap.h>
 #include <qwidget.h>
 
-class sizePreview : public QWidget
+class SimplePageSize;
+
+class SizePreview : public QWidget
 {
   Q_OBJECT
 
 public:
-  sizePreview( QWidget *parent, const char* name=0, WFlags f=0 );
+  SizePreview( QWidget *parent, const char* name=0, WFlags f=0 );
 
 public slots:
   // Sets the size.
-  void setSize(float width_mm, float height_mm);
+  void setSize(const SimplePageSize&);
 
   // or = 0 means "Portrait", anything else means "Landscape"
   void setOrientation(int ori); 
 
 protected:
-  void  paintEvent( QPaintEvent * );
+  void paintEvent(QPaintEvent*);
+  void resizeEvent(QResizeEvent*);
 
 private:
   int orientation; // 0 = portrait, other = landscape
@@ -37,6 +40,8 @@ private:
   // Both must be positive at all times!
   float _width; // in mm
   float _height; // in mm
+
+  QPixmap pixmap;
 };
 
 #endif
