@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qlabel.h>
@@ -81,12 +81,12 @@ public:
         m_autoSelThresh(0),
         m_dustsize(5),
         m_bgIsWhite(false),
-        m_sliderThresh(0L),
-        m_sliderDust(0L),
-        m_cbAutoSel(0L),
-        m_cbBackground(0L),
-        m_autoSelGroup(0L),
-        m_scanner(0L)
+        m_sliderThresh(0),
+        m_sliderDust(0),
+        m_cbAutoSel(0),
+        m_cbBackground(0),
+        m_autoSelGroup(0),
+        m_scanner(0)
         {
         }
     bool         m_doAutoSelection;  /* switch auto-selection on and off */
@@ -159,7 +159,7 @@ Previewer::Previewer(QWidget *parent, const char *name )
     bgroup = new QVButtonGroup( i18n("Scan Size"), this );
 
     // -----
-    pre_format_combo = new QComboBox( bgroup, "PREVIEWFORMATCOMBO" );
+    pre_format_combo = new QComboBox( this, "PREVIEWFORMATCOMBO" );
     pre_format_combo->insertItem( i18n( "Custom" ), ID_CUSTOM);
     pre_format_combo->insertItem( i18n( "DIN A4" ), ID_A4);
     pre_format_combo->insertItem( i18n( "DIN A5" ), ID_A5);
@@ -197,7 +197,7 @@ Previewer::Previewer(QWidget *parent, const char *name )
     d->m_autoSelGroup = new QGroupBox( 1, Horizontal, i18n("Auto-Selection"), this);
 
     QHBox *hbox       = new QHBox(d->m_autoSelGroup);
-    d->m_cbAutoSel    = new QCheckBox( i18n("active on"), hbox );
+    d->m_cbAutoSel    = new QCheckBox( i18n("Active on"), hbox );
     QToolTip::add( d->m_cbAutoSel, i18n("Check here if you want autodetection\n"
                                         "of the document on the preview."));
 
@@ -314,11 +314,8 @@ void Previewer::newImage( QImage *ni )
    m_previewImage = *ni;
 
    /* clear the auto detection arrays */
-   delete d->m_heightSum;
-   d->m_heightSum = 0;
-
-   delete d->m_widthSum;
-   d->m_widthSum = 0;
+   d->m_heightSum.resize( 0 );
+   d->m_widthSum.resize( 0 );
 
    img_canvas->newImage( &m_previewImage );
    findSelection( );
